#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

import ctypes, enum

from functools import wraps

from .macos_structs import label_t, ucred_t, POINTER64, fileglob_t, vnode_t


def init_ctx(f):
    @wraps(f)
    def wrapper(self, *args, **kw):
        if self.manager.label is None:
            label_addr = self.ql.os.heap.alloc(ctypes.sizeof(label_t))
            self.manager.label = label_t(self.ql, label_addr)
            self.manager.label.l_flags = 1
            self.manager.label.updateToMem()

        if self.manager.cred is None:
            cred_addr = self.ql.os.heap.alloc(ctypes.sizeof(ucred_t))
            self.manager.cred = ucred_t(self.ql, cred_addr)
            self.manager.cred.cr_ref = 2
            pyarr = [20, 12, 61, 79, 80, 81, 98, 33, 100, 204, 250, 395, 398, 399, 701, 0]
            self.manager.cred.cr_posix = ucred_t.posix_cred_t(
                501,
                501,
                501,
                15,
                (ctypes.c_uint32 * len(pyarr))(*pyarr),
                20,
                20,
                501,
                2)
            self.manager.cred.cr_label = POINTER64(self.manager.label.base)
            self.manager.cred.updateToMem()

        if self.manager.vnode is None:
            tmp_addr = self.ql.os.heap.alloc(ctypes.sizeof(vnode_t))
            self.manager.vnode = vnode_t(self.ql, tmp_addr)
            tmp_name = self.ql.os.heap.alloc(len(self.manager.current_proc))
            self.manager.ql.mem.write(tmp_name, self.manager.current_proc.encode())
            self.manager.vnode.v_name = POINTER64(tmp_name)
            self.manager.vnode.updateToMem()

        return f(self, *args, **kw)
    return wrapper

class QlMacOSPolicy():
    def __init__(self, ql, manager):
        self.ql = ql
        self.manager = manager
        self.bank = {}

    @init_ctx
    def mpo_file_check_mmap(self, prot, flags, file_pos):
        if self.manager.vnode is None or self.manager.cred is None:
            ql.log.info("Invalid vnode or credential")
            return
        fg_addr = self.ql.os.heap.alloc(ctypes.sizeof(fileglob_t))
        fg = fileglob_t(self.ql, fg_addr)
        fg.fg_flag = 1
        fg.fg_count = 1
        fg.fg_cred = POINTER64(self.manager.cred.base)
        fg.fg_data = POINTER64(self.manager.vnode.base)
        fg.updateToMem()

        self.manager.emit_by_type(MACPolicy_EventType.EV_MAC_mpo_file_check_mmap, 
            [self.manager.cred.base, fg_addr, self.manager.label.base, prot, flags, file_pos, 0])

base_event_MAC = 0x2000

class AutoNumber(enum.Enum):
     def __new__(cls):
        value = len(cls.__members__) + base_event_MAC
        obj = object.__new__(cls)
        obj._value_ = value
        return obj

NUM_EVENT_MAC_POLICY = 335

class MACPolicy_EventType(AutoNumber):
    EV_MAC_mpo_audit_check_postselect = ()
    EV_MAC_mpo_audit_check_preselect = ()
    EV_MAC_mpo_bpfdesc_label_associate = ()
    EV_MAC_mpo_bpfdesc_label_destroy = ()
    EV_MAC_mpo_bpfdesc_label_init = ()
    EV_MAC_mpo_bpfdesc_check_receive = ()
    EV_MAC_mpo_cred_check_label_update_execve = ()
    EV_MAC_mpo_cred_check_label_update = ()
    EV_MAC_mpo_cred_check_visible = ()
    EV_MAC_mpo_cred_label_associate_fork = ()
    EV_MAC_mpo_cred_label_associate_kernel = ()
    EV_MAC_mpo_cred_label_associate = ()
    EV_MAC_mpo_cred_label_associate_user = ()
    EV_MAC_mpo_cred_label_destroy = ()
    EV_MAC_mpo_cred_label_externalize_audit = ()
    EV_MAC_mpo_cred_label_externalize = ()
    EV_MAC_mpo_cred_label_init = ()
    EV_MAC_mpo_cred_label_internalize = ()
    EV_MAC_mpo_cred_label_update_execve = ()
    EV_MAC_mpo_cred_label_update = ()
    EV_MAC_mpo_devfs_label_associate_device = ()
    EV_MAC_mpo_devfs_label_associate_directory = ()
    EV_MAC_mpo_devfs_label_copy = ()
    EV_MAC_mpo_devfs_label_destroy = ()
    EV_MAC_mpo_devfs_label_init = ()
    EV_MAC_mpo_devfs_label_update = ()
    EV_MAC_mpo_file_check_change_offset = ()
    EV_MAC_mpo_file_check_create = ()
    EV_MAC_mpo_file_check_dup = ()
    EV_MAC_mpo_file_check_fcntl = ()
    EV_MAC_mpo_file_check_get_offset = ()
    EV_MAC_mpo_file_check_get = ()
    EV_MAC_mpo_file_check_inherit = ()
    EV_MAC_mpo_file_check_ioctl = ()
    EV_MAC_mpo_file_check_lock = ()
    EV_MAC_mpo_file_check_mmap_downgrade = ()
    EV_MAC_mpo_file_check_mmap = ()
    EV_MAC_mpo_file_check_receive = ()
    EV_MAC_mpo_file_check_set = ()
    EV_MAC_mpo_file_label_init = ()
    EV_MAC_mpo_file_label_destroy = ()
    EV_MAC_mpo_file_label_associate = ()
    EV_MAC_mpo_ifnet_check_label_update = ()
    EV_MAC_mpo_ifnet_check_transmit = ()
    EV_MAC_mpo_ifnet_label_associate = ()
    EV_MAC_mpo_ifnet_label_copy = ()
    EV_MAC_mpo_ifnet_label_destroy = ()
    EV_MAC_mpo_ifnet_label_externalize = ()
    EV_MAC_mpo_ifnet_label_init = ()
    EV_MAC_mpo_ifnet_label_internalize = ()
    EV_MAC_mpo_ifnet_label_update = ()
    EV_MAC_mpo_ifnet_label_recycle = ()
    EV_MAC_mpo_inpcb_check_deliver = ()
    EV_MAC_mpo_inpcb_label_associate = ()
    EV_MAC_mpo_inpcb_label_destroy = ()
    EV_MAC_mpo_inpcb_label_init = ()
    EV_MAC_mpo_inpcb_label_recycle = ()
    EV_MAC_mpo_inpcb_label_update = ()
    EV_MAC_mpo_iokit_check_device = ()
    EV_MAC_mpo_ipq_label_associate = ()
    EV_MAC_mpo_ipq_label_compare = ()
    EV_MAC_mpo_ipq_label_destroy = ()
    EV_MAC_mpo_ipq_label_init = ()
    EV_MAC_mpo_ipq_label_update = ()
    EV_MAC_mpo_file_check_library_validation = ()
    EV_MAC_mpo_vnode_notify_setacl = ()
    EV_MAC_mpo_vnode_notify_setattrlist = ()
    EV_MAC_mpo_vnode_notify_setextattr = ()
    EV_MAC_mpo_vnode_notify_setflags = ()
    EV_MAC_mpo_vnode_notify_setmode = ()
    EV_MAC_mpo_vnode_notify_setowner = ()
    EV_MAC_mpo_vnode_notify_setutimes = ()
    EV_MAC_mpo_vnode_notify_truncate = ()
    EV_MAC_mpo_mbuf_label_associate_bpfdesc = ()
    EV_MAC_mpo_mbuf_label_associate_ifnet = ()
    EV_MAC_mpo_mbuf_label_associate_inpcb = ()
    EV_MAC_mpo_mbuf_label_associate_ipq = ()
    EV_MAC_mpo_mbuf_label_associate_linklayer = ()
    EV_MAC_mpo_mbuf_label_associate_multicast_encap = ()
    EV_MAC_mpo_mbuf_label_associate_netlayer = ()
    EV_MAC_mpo_mbuf_label_associate_socket = ()
    EV_MAC_mpo_mbuf_label_copy = ()
    EV_MAC_mpo_mbuf_label_destroy = ()
    EV_MAC_mpo_mbuf_label_init = ()
    EV_MAC_mpo_mount_check_fsctl = ()
    EV_MAC_mpo_mount_check_getattr = ()
    EV_MAC_mpo_mount_check_label_update = ()
    EV_MAC_mpo_mount_check_mount = ()
    EV_MAC_mpo_mount_check_remount = ()
    EV_MAC_mpo_mount_check_setattr = ()
    EV_MAC_mpo_mount_check_stat = ()
    EV_MAC_mpo_mount_check_umount = ()
    EV_MAC_mpo_mount_label_associate = ()
    EV_MAC_mpo_mount_label_destroy = ()
    EV_MAC_mpo_mount_label_externalize = ()
    EV_MAC_mpo_mount_label_init = ()
    EV_MAC_mpo_mount_label_internalize = ()
    EV_MAC_mpo_netinet_fragment = ()
    EV_MAC_mpo_netinet_icmp_reply = ()
    EV_MAC_mpo_netinet_tcp_reply = ()
    EV_MAC_mpo_pipe_check_ioctl = ()
    EV_MAC_mpo_pipe_check_kqfilter = ()
    EV_MAC_mpo_pipe_check_label_update = ()
    EV_MAC_mpo_pipe_check_read = ()
    EV_MAC_mpo_pipe_check_select = ()
    EV_MAC_mpo_pipe_check_stat = ()
    EV_MAC_mpo_pipe_check_write = ()
    EV_MAC_mpo_pipe_label_associate = ()
    EV_MAC_mpo_pipe_label_copy = ()
    EV_MAC_mpo_pipe_label_destroy = ()
    EV_MAC_mpo_pipe_label_externalize = ()
    EV_MAC_mpo_pipe_label_init = ()
    EV_MAC_mpo_pipe_label_internalize = ()
    EV_MAC_mpo_pipe_label_update = ()
    EV_MAC_mpo_policy_destroy = ()
    EV_MAC_mpo_policy_init = ()
    EV_MAC_mpo_policy_initbsd = ()
    EV_MAC_mpo_policy_syscall = ()
    EV_MAC_mpo_system_check_sysctlbyname = ()
    EV_MAC_mpo_proc_check_inherit_ipc_ports = ()
    EV_MAC_mpo_vnode_check_rename = ()
    EV_MAC_mpo_kext_check_query = ()
    EV_MAC_mpo_proc_notify_exec_complete = ()
    EV_MAC_mpo_reserved5 = ()
    EV_MAC_mpo_reserved6 = ()
    EV_MAC_mpo_proc_check_expose_task = ()
    EV_MAC_mpo_proc_check_set_host_special_port = ()
    EV_MAC_mpo_proc_check_set_host_exception_port = ()
    EV_MAC_mpo_exc_action_check_exception_send = ()
    EV_MAC_mpo_exc_action_label_associate = ()
    EV_MAC_mpo_exc_action_label_populate = ()
    EV_MAC_mpo_exc_action_label_destroy = ()
    EV_MAC_mpo_exc_action_label_init = ()
    EV_MAC_mpo_exc_action_label_update = ()
    EV_MAC_mpo_vnode_check_trigger_resolve = ()
    EV_MAC_mpo_reserved1 = ()
    EV_MAC_mpo_reserved2 = ()
    EV_MAC_mpo_reserved3 = ()
    EV_MAC_mpo_skywalk_flow_check_connect = ()
    EV_MAC_mpo_skywalk_flow_check_listen = ()
    EV_MAC_mpo_posixsem_check_create = ()
    EV_MAC_mpo_posixsem_check_open = ()
    EV_MAC_mpo_posixsem_check_post = ()
    EV_MAC_mpo_posixsem_check_unlink = ()
    EV_MAC_mpo_posixsem_check_wait = ()
    EV_MAC_mpo_posixsem_label_associate = ()
    EV_MAC_mpo_posixsem_label_destroy = ()
    EV_MAC_mpo_posixsem_label_init = ()
    EV_MAC_mpo_posixshm_check_create = ()
    EV_MAC_mpo_posixshm_check_mmap = ()
    EV_MAC_mpo_posixshm_check_open = ()
    EV_MAC_mpo_posixshm_check_stat = ()
    EV_MAC_mpo_posixshm_check_truncate = ()
    EV_MAC_mpo_posixshm_check_unlink = ()
    EV_MAC_mpo_posixshm_label_associate = ()
    EV_MAC_mpo_posixshm_label_destroy = ()
    EV_MAC_mpo_posixshm_label_init = ()
    EV_MAC_mpo_proc_check_debug = ()
    EV_MAC_mpo_proc_check_fork = ()
    EV_MAC_mpo_proc_check_get_task_name = ()
    EV_MAC_mpo_proc_check_get_task = ()
    EV_MAC_mpo_proc_check_getaudit = ()
    EV_MAC_mpo_proc_check_getauid = ()
    EV_MAC_mpo_proc_check_getlcid = ()
    EV_MAC_mpo_proc_check_mprotect = ()
    EV_MAC_mpo_proc_check_sched = ()
    EV_MAC_mpo_proc_check_setaudit = ()
    EV_MAC_mpo_proc_check_setauid = ()
    EV_MAC_mpo_proc_check_setlcid = ()
    EV_MAC_mpo_proc_check_signal = ()
    EV_MAC_mpo_proc_check_wait = ()
    EV_MAC_mpo_proc_label_destroy = ()
    EV_MAC_mpo_proc_label_init = ()
    EV_MAC_mpo_socket_check_accept = ()
    EV_MAC_mpo_socket_check_accepted = ()
    EV_MAC_mpo_socket_check_bind = ()
    EV_MAC_mpo_socket_check_connect = ()
    EV_MAC_mpo_socket_check_create = ()
    EV_MAC_mpo_socket_check_deliver = ()
    EV_MAC_mpo_socket_check_kqfilter = ()
    EV_MAC_mpo_socket_check_label_update = ()
    EV_MAC_mpo_socket_check_listen = ()
    EV_MAC_mpo_socket_check_receive = ()
    EV_MAC_mpo_socket_check_received = ()
    EV_MAC_mpo_socket_check_select = ()
    EV_MAC_mpo_socket_check_send = ()
    EV_MAC_mpo_socket_check_stat = ()
    EV_MAC_mpo_socket_check_setsockopt = ()
    EV_MAC_mpo_socket_check_getsockopt = ()
    EV_MAC_mpo_socket_label_associate_accept = ()
    EV_MAC_mpo_socket_label_associate = ()
    EV_MAC_mpo_socket_label_copy = ()
    EV_MAC_mpo_socket_label_destroy = ()
    EV_MAC_mpo_socket_label_externalize = ()
    EV_MAC_mpo_socket_label_init = ()
    EV_MAC_mpo_socket_label_internalize = ()
    EV_MAC_mpo_socket_label_update = ()
    EV_MAC_mpo_socketpeer_label_associate_mbuf = ()
    EV_MAC_mpo_socketpeer_label_associate_socket = ()
    EV_MAC_mpo_socketpeer_label_destroy = ()
    EV_MAC_mpo_socketpeer_label_externalize = ()
    EV_MAC_mpo_socketpeer_label_init = ()
    EV_MAC_mpo_system_check_acct = ()
    EV_MAC_mpo_system_check_audit = ()
    EV_MAC_mpo_system_check_auditctl = ()
    EV_MAC_mpo_system_check_auditon = ()
    EV_MAC_mpo_system_check_host_priv = ()
    EV_MAC_mpo_system_check_nfsd = ()
    EV_MAC_mpo_system_check_reboot = ()
    EV_MAC_mpo_system_check_settime = ()
    EV_MAC_mpo_system_check_swapoff = ()
    EV_MAC_mpo_system_check_swapon = ()
    EV_MAC_mpo_socket_check_ioctl = ()
    EV_MAC_mpo_sysvmsg_label_associate = ()
    EV_MAC_mpo_sysvmsg_label_destroy = ()
    EV_MAC_mpo_sysvmsg_label_init = ()
    EV_MAC_mpo_sysvmsg_label_recycle = ()
    EV_MAC_mpo_sysvmsq_check_enqueue = ()
    EV_MAC_mpo_sysvmsq_check_msgrcv = ()
    EV_MAC_mpo_sysvmsq_check_msgrmid = ()
    EV_MAC_mpo_sysvmsq_check_msqctl = ()
    EV_MAC_mpo_sysvmsq_check_msqget = ()
    EV_MAC_mpo_sysvmsq_check_msqrcv = ()
    EV_MAC_mpo_sysvmsq_check_msqsnd = ()
    EV_MAC_mpo_sysvmsq_label_associate = ()
    EV_MAC_mpo_sysvmsq_label_destroy = ()
    EV_MAC_mpo_sysvmsq_label_init = ()
    EV_MAC_mpo_sysvmsq_label_recycle = ()
    EV_MAC_mpo_sysvsem_check_semctl = ()
    EV_MAC_mpo_sysvsem_check_semget = ()
    EV_MAC_mpo_sysvsem_check_semop = ()
    EV_MAC_mpo_sysvsem_label_associate = ()
    EV_MAC_mpo_sysvsem_label_destroy = ()
    EV_MAC_mpo_sysvsem_label_init = ()
    EV_MAC_mpo_sysvsem_label_recycle = ()
    EV_MAC_mpo_sysvshm_check_shmat = ()
    EV_MAC_mpo_sysvshm_check_shmctl = ()
    EV_MAC_mpo_sysvshm_check_shmdt = ()
    EV_MAC_mpo_sysvshm_check_shmget = ()
    EV_MAC_mpo_sysvshm_label_associate = ()
    EV_MAC_mpo_sysvshm_label_destroy = ()
    EV_MAC_mpo_sysvshm_label_init = ()
    EV_MAC_mpo_sysvshm_label_recycle = ()
    EV_MAC_mpo_proc_notify_exit = ()
    EV_MAC_mpo_mount_check_snapshot_revert = ()
    EV_MAC_mpo_vnode_check_getattr = ()
    EV_MAC_mpo_mount_check_snapshot_create = ()
    EV_MAC_mpo_mount_check_snapshot_delete = ()
    EV_MAC_mpo_vnode_check_clone = ()
    EV_MAC_mpo_proc_check_get_cs_info = ()
    EV_MAC_mpo_proc_check_set_cs_info = ()
    EV_MAC_mpo_iokit_check_hid_control = ()
    EV_MAC_mpo_vnode_check_access = ()
    EV_MAC_mpo_vnode_check_chdir = ()
    EV_MAC_mpo_vnode_check_chroot = ()
    EV_MAC_mpo_vnode_check_create = ()
    EV_MAC_mpo_vnode_check_deleteextattr = ()
    EV_MAC_mpo_vnode_check_exchangedata = ()
    EV_MAC_mpo_vnode_check_exec = ()
    EV_MAC_mpo_vnode_check_getattrlist = ()
    EV_MAC_mpo_vnode_check_getextattr = ()
    EV_MAC_mpo_vnode_check_ioctl = ()
    EV_MAC_mpo_vnode_check_kqfilter = ()
    EV_MAC_mpo_vnode_check_label_update = ()
    EV_MAC_mpo_vnode_check_link = ()
    EV_MAC_mpo_vnode_check_listextattr = ()
    EV_MAC_mpo_vnode_check_lookup = ()
    EV_MAC_mpo_vnode_check_open = ()
    EV_MAC_mpo_vnode_check_read = ()
    EV_MAC_mpo_vnode_check_readdir = ()
    EV_MAC_mpo_vnode_check_readlink = ()
    EV_MAC_mpo_vnode_check_rename_from = ()
    EV_MAC_mpo_vnode_check_rename_to = ()
    EV_MAC_mpo_vnode_check_revoke = ()
    EV_MAC_mpo_vnode_check_select = ()
    EV_MAC_mpo_vnode_check_setattrlist = ()
    EV_MAC_mpo_vnode_check_setextattr = ()
    EV_MAC_mpo_vnode_check_setflags = ()
    EV_MAC_mpo_vnode_check_setmode = ()
    EV_MAC_mpo_vnode_check_setowner = ()
    EV_MAC_mpo_vnode_check_setutimes = ()
    EV_MAC_mpo_vnode_check_stat = ()
    EV_MAC_mpo_vnode_check_truncate = ()
    EV_MAC_mpo_vnode_check_unlink = ()
    EV_MAC_mpo_vnode_check_write = ()
    EV_MAC_mpo_vnode_label_associate_devfs = ()
    EV_MAC_mpo_vnode_label_associate_extattr = ()
    EV_MAC_mpo_vnode_label_associate_file = ()
    EV_MAC_mpo_vnode_label_associate_pipe = ()
    EV_MAC_mpo_vnode_label_associate_posixsem = ()
    EV_MAC_mpo_vnode_label_associate_posixshm = ()
    EV_MAC_mpo_vnode_label_associate_singlelabel = ()
    EV_MAC_mpo_vnode_label_associate_socket = ()
    EV_MAC_mpo_vnode_label_copy = ()
    EV_MAC_mpo_vnode_label_destroy = ()
    EV_MAC_mpo_vnode_label_externalize_audit = ()
    EV_MAC_mpo_vnode_label_externalize = ()
    EV_MAC_mpo_vnode_label_init = ()
    EV_MAC_mpo_vnode_label_internalize = ()
    EV_MAC_mpo_vnode_label_recycle = ()
    EV_MAC_mpo_vnode_label_store = ()
    EV_MAC_mpo_vnode_label_update_extattr = ()
    EV_MAC_mpo_vnode_label_update = ()
    EV_MAC_mpo_vnode_notify_create = ()
    EV_MAC_mpo_vnode_check_signature = ()
    EV_MAC_mpo_vnode_check_uipc_bind = ()
    EV_MAC_mpo_vnode_check_uipc_connect = ()
    EV_MAC_mpo_proc_check_run_cs_invalid = ()
    EV_MAC_mpo_proc_check_suspend_resume = ()
    EV_MAC_mpo_thread_userret = ()
    EV_MAC_mpo_iokit_check_set_properties = ()
    EV_MAC_mpo_system_check_chud = ()
    EV_MAC_mpo_vnode_check_searchfs = ()
    EV_MAC_mpo_priv_check = ()
    EV_MAC_mpo_priv_grant = ()
    EV_MAC_mpo_proc_check_map_anon = ()
    EV_MAC_mpo_vnode_check_fsgetpath = ()
    EV_MAC_mpo_iokit_check_open = ()
    EV_MAC_mpo_proc_check_ledger = ()
    EV_MAC_mpo_vnode_notify_rename = ()
    EV_MAC_mpo_vnode_check_setacl = ()
    EV_MAC_mpo_vnode_notify_deleteextattr = ()
    EV_MAC_mpo_system_check_kas_info = ()
    EV_MAC_mpo_vnode_check_lookup_preflight = ()
    EV_MAC_mpo_vnode_notify_open = ()
    EV_MAC_mpo_system_check_info = ()
    EV_MAC_mpo_pty_notify_grant = ()
    EV_MAC_mpo_pty_notify_close = ()
    EV_MAC_mpo_vnode_find_sigs = ()
    EV_MAC_mpo_kext_check_load = ()
    EV_MAC_mpo_kext_check_unload = ()
    EV_MAC_mpo_proc_check_proc_info = ()
    EV_MAC_mpo_vnode_notify_link = ()
    EV_MAC_mpo_iokit_check_filter_properties = ()
    EV_MAC_mpo_iokit_check_get_property = ()



#!/usr/bin/env python3
#
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

import struct
import time
from qiling.os.windows.const import *
from qiling.os.const import *
from qiling.os.windows.fncc import *
from qiling.os.windows.utils import *
from qiling.os.windows.thread import *
from qiling.os.windows.handle import *
from qiling.exception import *


dllname = 'kernel32_dll'

# BOOL QueryPerformanceCounter(
#   LARGE_INTEGER *lpPerformanceCount
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_QueryPerformanceCounter(ql, address, params):
    ret = 0
    return ret


# BOOL QueryPerformanceFrequency(
#  LARGE_INTEGER *lpFrequency
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_QueryPerformanceFrequency(ql, address, params):
    ql.mem.write(params['lpFrequency'], (10000000).to_bytes(length=8, byteorder='little'))
    return 1



#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

# Subsystems which program may communicate to 
# generated by MIG
# We can find the defines in /usr/include/mach/*.defs
# Now we havent finish the server it self, only return the fixed data
# TODO: finish the servers
# find Release source code in https://github.com/doadam/xnu-4570.1.46/master/BUILD/obj/RELEASE_X86_64/osfmk/RELEASE/mach/mach_host_server.c

from struct import pack, unpack

from qiling.const import *
from .mach_port import *
from .const import *

class MachHostServer():

    def __init__(self, ql):
        self.ql = ql
        self.system_rpc = {
            200: self.host_info,
            206: self.host_get_clock_service
        }
        pass

    def host_info(self, in_header, in_content):
        # parse request
        out_msg = MachMsg(self.ql)
        if len(in_content) < 16:
            self.ql.log.debug("Error in Host info SubSystem -hostinfo()")
            raise
        ndr = unpack("<Q", in_content[:8])[0]
        flavor = unpack("<L", in_content[8:12])[0]
        host_info_outCnt = unpack("<L", in_content[12:16])[0]
        # may be we need wapper a func in kernel func ,but it is also a huge projrct 
        # gen reply

        if flavor == HOST_BASIC_INFO:
            out_msg.header.msgh_bits = 4608
            out_msg.header.msgh_size = 88
            out_msg.header.msgh_remote_port = 0
            out_msg.header.msgh_local_port = self.ql.os.macho_mach_port.name
            out_msg.header.msgh_voucher_port = 0
            out_msg.header.msgh_id = 300

            out_msg.content += pack("<Q", 0x100000000)      # NDR
            out_msg.content += pack("<L", 0x0)              # ret code / KERN SUCCESS
            out_msg.content += pack("<L", host_info_outCnt) # host info output count 

            # host info out
            out_msg.content += pack("<L", 0x2)              # max_cpus
            out_msg.content += pack("<L", 0x2)              # avail_cpus
            out_msg.content += pack("<L", 0x80000000)       # memory_size
            out_msg.content += pack("<L", 0x7)              # cpu_type
            out_msg.content += pack("<L", 0x4)              # cpu_subtype
            if host_info_outCnt > 5:
                out_msg.content += pack("<L", 0x1)          # cpu_threadtype
                out_msg.content += pack("<L", 0x2)          # physical_cpu
                out_msg.content += pack("<L", 0x2)          # physical_cpu_max
                out_msg.content += pack("<L", 0x4)          # logic_cpu
                out_msg.content += pack("<L", 0x4)          # logic_cpu_max
                out_msg.content += pack("<Q", 0x400000000)  # max_mem
            
        elif flavor == HOST_PRIORITY_INFO:
            out_msg.header.msgh_bits = 4608
            out_msg.header.msgh_size = 72
            out_msg.header.msgh_remote_port = 0
            out_msg.header.msgh_local_port = self.ql.os.macho_mach_port.name
            out_msg.header.msgh_voucher_port = 0
            out_msg.header.msgh_id = 300

            out_msg.content += pack("<Q", 0x100000000)      # NDR
            out_msg.content += pack("<L", 0x0)              # ret code / KERN SUCCESS
            out_msg.content += pack("<L", host_info_outCnt) # host info output count 
            # host info out
            out_msg.content += pack("<L", 0x50)             # kernel_priority = MINPRI_KERNEL;
            out_msg.content += pack("<L", 0x50)             # system_priority = MINPRI_KERNEL;
            out_msg.content += pack("<L", 0x40)             # server_priority = MINPRI_RESERVED;
            out_msg.content += pack("<L", 0x1f)             # user_priority = BASEPRI_DEFAULT;
            out_msg.content += pack("<L", 0x0)              # depress_priority = DEPRESSPRI;
            out_msg.content += pack("<L", 0x0)              # idle_priority = IDLEPRI;
            out_msg.content += pack("<L", 0x0)              # minimum_priority = MINPRI_USER;
            out_msg.content += pack("<L", 0x4f)             # maximum_priority = MAXPRI_RESERVED
        else:
            self.ql.log.debug("Host flavor not support")
            raise
        return out_msg

    def host_get_clock_service(self, in_header, in_content):
        out_msg = MachMsg(self.ql)

        out_msg.header.msgh_bits = 0x80001200
        out_msg.header.msgh_size = 0x00000028
        out_msg.header.msgh_remote_port = 0x00000000
        out_msg.header.msgh_local_port = self.ql.os.macho_mach_port.name
        out_msg.header.msgh_voucher_port = 0
        out_msg.header.msgh_id = 306

        out_msg.content = pack("<L", 0x1)

        out_msg.trailer = b''
        out_msg.trailer += pack("<L", self.ql.os.macho_port_manager.clock_port.name)       # clock port name 
        out_msg.trailer += pack("<L", 0x0)                                              # pad1
        out_msg.trailer += pack("<H", 0x0)                                              # pad2
        out_msg.trailer += pack("<B", 0x11)                                             # disposition
        out_msg.trailer += pack("<B", 0x0)                                              # type
        out_msg.trailer += pack("<L", 0x0)                                              # pad end
        
        return out_msg


class MachTaskServer():

    def __init__(self, ql):
        self.ql = ql

    def semaphore_create(self, in_header, in_content):
        out_msg = MachMsg(self.ql)

        out_msg.header.msgh_bits = 0x80001200
        out_msg.header.msgh_size = 0x00000028
        out_msg.header.msgh_remote_port = 0x00000000
        out_msg.header.msgh_local_port = self.ql.os.macho_mach_port.name
        out_msg.header.msgh_voucher_port = 0
        out_msg.header.msgh_id = 3518

        out_msg.content = pack("<L", 0x1)

        out_msg.trailer = b''
        out_msg.trailer += pack("<L", self.ql.os.macho_port_manager.semaphore_port.name)       # clock port name 
        out_msg.trailer += pack("<L", 0x0)                                                  # pad1
        out_msg.trailer += pack("<H", 0x0)                                                  # pad2
        out_msg.trailer += pack("<B", 0x11)                                                 # disposition
        out_msg.trailer += pack("<B", 0x0)                                                  # type
        out_msg.trailer += pack("<L", 0x0)                                                  # pad end

        return out_msg

    def get_special_port(self, in_header, in_content):
        out_msg = MachMsg(self.ql)
        out_msg.header.msgh_bits = 0x80001200
        out_msg.header.msgh_size = 0x00000028
        out_msg.header.msgh_remote_port = 0x00000000
        out_msg.header.msgh_local_port = self.ql.os.macho_mach_port.name
        out_msg.header.msgh_voucher_port = 0
        out_msg.header.msgh_id = 3509

        out_msg.content = pack("<L", 0x1)

        out_msg.trailer = b''
        out_msg.trailer += pack("<L", self.ql.os.macho_port_manager.special_port.name)         # special port name 
        out_msg.trailer += pack("<L", 0x0)                                                  # pad1
        out_msg.trailer += pack("<H", 0x0)                                                  # pad2
        out_msg.trailer += pack("<B", 0x11)                                                 # disposition
        out_msg.trailer += pack("<B", 0x0)                                                  # type
        out_msg.trailer += pack("<L", 0x0)                                                  # pad end

        return out_msg
        pass



#!/usr/bin/env python3
#
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#


from qiling.const import *
from qiling.os.linux.thread import *
from qiling.const import *
from qiling.os.posix.filestruct import *
from qiling.os.filestruct import *
from qiling.os.posix.const_mapping import *
from qiling.exception import *

from qiling.os.posix.syscall.socket import ql_syscall_socket, ql_syscall_connect, ql_syscall_recv, ql_syscall_send, ql_syscall_bind, ql_syscall_listen, ql_syscall_accept, ql_syscall_setsockopt, ql_syscall_recvfrom, ql_syscall_sendto

def ql_syscall_socketcall(ql, socketcall_call, socketcall_args, *args, **kw):
    SOCKETCALL_SYS_SOCKET = 1
    SOCKETCALL_SYS_BIND = 2
    SOCKETCALL_SYS_CONNECT = 3
    SOCKETCALL_SYS_LISTEN = 4
    SOCKETCALL_SYS_ACCEPT = 5
    SOCKETCALL_SYS_GETSOCKNAME = 6 
    SOCKETCALL_SYS_GETPEERNAME = 7
    SOCKETCALL_SYS_SOCKETPAIR = 8
    SOCKETCALL_SYS_SEND = 9
    SOCKETCALL_SYS_RECV = 10
    SOCKETCALL_SYS_SENDTO = 11
    SOCKETCALL_SYS_RECVFROM = 12
    SOCKETCALL_SYS_SHUTDOWN = 13
    SOCKETCALL_SYS_SETSOCKOPT = 14
    SOCKETCALL_SYS_GETSOCKOPT = 15
    SOCKETCALL_SYS_SENDMSG = 16
    SOCKETCALL_SYS_RECVMSG = 17
    SOCKETCALL_SYS_ACCEPT4 = 18
    SOCKETCALL_SYS_RECVMMSG = 19
    SOCKETCALL_SYS_SENDMMSG = 20

    if socketcall_call == SOCKETCALL_SYS_SOCKET:
        socketcall_domain = ql.unpack(ql.mem.read(socketcall_args, ql.pointersize))
        socketcall_type = ql.unpack(ql.mem.read(socketcall_args + ql.pointersize, ql.pointersize))
        socketcall_protocol = ql.unpack(ql.mem.read(socketcall_args + ql.pointersize * 2, ql.pointersize))
        return ql_syscall_socket(ql, socketcall_domain, socketcall_type, socketcall_protocol, 0, 0, 0)
    
    elif socketcall_call == SOCKETCALL_SYS_CONNECT:
        socketcall_sockfd = ql.unpack(ql.mem.read(socketcall_args, ql.pointersize))
        socketcall_addr = ql.unpack(ql.mem.read(socketcall_args + ql.pointersize, ql.pointersize))
        socketcall_addrlen = ql.unpack(ql.mem.read(socketcall_args + ql.pointersize * 2, ql.pointersize))
        return ql_syscall_connect(ql, socketcall_sockfd, socketcall_addr, socketcall_addrlen, 0, 0, 0)
    
    elif socketcall_call == SOCKETCALL_SYS_SEND:
        socketcall_sockfd = ql.unpack(ql.mem.read(socketcall_args, ql.pointersize))
        socketcall_buf = ql.unpack(ql.mem.read(socketcall_args + ql.pointersize, ql.pointersize))
        socketcall_len = ql.unpack(ql.mem.read(socketcall_args + ql.pointersize * 2, ql.pointersize))
        socketcall_flags = ql.unpack(ql.mem.read(socketcall_args + ql.pointersize * 3, ql.pointersize))
        return ql_syscall_send(ql, socketcall_sockfd, socketcall_buf, socketcall_len, socketcall_flags, 0, 0)

    elif socketcall_call == SOCKETCALL_SYS_RECVFROM:
        socketcall_sockfd = ql.unpack(ql.mem.read(socketcall_args, ql.pointersize))
        socketcall_buf = ql.unpack(ql.mem.read(socketcall_args + ql.pointersize, ql.pointersize))
        socketcall_len = ql.unpack(ql.mem.read(socketcall_args + ql.pointersize * 2, ql.pointersize))
        socketcall_flags = ql.unpack(ql.mem.read(socketcall_args + ql.pointersize * 3, ql.pointersize))
        socketcall_addr = ql.unpack(ql.mem.read(socketcall_args + ql.pointersize * 4, ql.pointersize))
        socketcall_addrlen = ql.unpack(ql.mem.read(socketcall_args + ql.pointersize * 5, ql.pointersize))
        return ql_syscall_recvfrom(ql, socketcall_sockfd, socketcall_buf, socketcall_len, socketcall_flags, socketcall_addr, socketcall_addrlen)

    elif socketcall_call == SOCKETCALL_SYS_SENDTO:
        socketcall_sockfd = ql.unpack(ql.mem.read(socketcall_args, ql.pointersize))
        socketcall_buf = ql.unpack(ql.mem.read(socketcall_args + ql.pointersize, ql.pointersize))
        socketcall_len = ql.unpack(ql.mem.read(socketcall_args + ql.pointersize * 2, ql.pointersize))
        socketcall_flags = ql.unpack(ql.mem.read(socketcall_args + ql.pointersize * 3, ql.pointersize))
        socketcall_addr = ql.unpack(ql.mem.read(socketcall_args + ql.pointersize * 4, ql.pointersize))
        socketcall_addrlen = ql.unpack(ql.mem.read(socketcall_args + ql.pointersize * 5, ql.pointersize))
        return ql_syscall_sendto(ql, socketcall_sockfd, socketcall_buf, socketcall_len, socketcall_flags, socketcall_addr, socketcall_addrlen)

    elif socketcall_call == SOCKETCALL_SYS_RECV:
        socketcall_sockfd = ql.unpack(ql.mem.read(socketcall_args, ql.pointersize))
        socketcall_buf = ql.unpack(ql.mem.read(socketcall_args + ql.pointersize, ql.pointersize))
        socketcall_len = ql.unpack(ql.mem.read(socketcall_args + ql.pointersize * 2, ql.pointersize))
        socketcall_flags = ql.unpack(ql.mem.read(socketcall_args + ql.pointersize * 3, ql.pointersize))
        return ql_syscall_recv(ql, socketcall_sockfd, socketcall_buf, socketcall_len, socketcall_flags, 0, 0)
    
    elif socketcall_call == SOCKETCALL_SYS_BIND:
        socketcall_fd = ql.unpack(ql.mem.read(socketcall_args, ql.pointersize))
        socketcall_addr = ql.unpack(ql.mem.read(socketcall_args + ql.pointersize, ql.pointersize))
        socketcall_addrlen = ql.unpack(ql.mem.read(socketcall_args + ql.pointersize * 2, ql.pointersize))
        return ql_syscall_bind(ql, socketcall_fd, socketcall_addr, socketcall_addrlen)
    
    elif socketcall_call == SOCKETCALL_SYS_LISTEN:
        socketcall_sockfd = ql.unpack(ql.mem.read(socketcall_args, ql.pointersize))
        socketcall_backlog = ql.unpack(ql.mem.read(socketcall_args + ql.pointersize, ql.pointersize))
        return ql_syscall_listen(ql, socketcall_sockfd, socketcall_backlog)
    
    elif socketcall_call == SOCKETCALL_SYS_ACCEPT:
        socketcall_sockfd = ql.unpack(ql.mem.read(socketcall_args, ql.pointersize))
        socketcall_addr = ql.unpack(ql.mem.read(socketcall_args + ql.pointersize, ql.pointersize))
        socketcall_addrlen = ql.unpack(ql.mem.read(socketcall_args + ql.pointersize * 2, ql.pointersize))
        return ql_syscall_accept(ql, socketcall_sockfd, socketcall_addr, socketcall_addrlen)
    
    elif socketcall_call == SOCKETCALL_SYS_SETSOCKOPT:
        return ql_syscall_setsockopt(ql)

    else:
        ql.log.debug("error call %d" % socketcall_call)
        ql.os.stop(stop_event=THREAD_EVENT_UNEXECPT_EVENT)
        raise



#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

from qiling.const import *
from qiling.exception import *

DWORD = 1
UINT = 1
INT = 1
BOOL = 1
SIZE_T = 1
BYTE = 1
ULONGLONG = 2
HANDLE = 3
POINTER = 3
STRING = 4
WSTRING = 5

def linux_kernel_api(param_num=None, params=None):
    """
    @cc: windows api calling convention, only x86 needs this, x64 is always fastcall
    @params: params dict
    @param_num: the number of function params, used by variadic functions, e.g printf
    """
    def decorator(func):
        def wrapper(*args, **kwargs):
            ql = args[0]
            if ql.archtype == QL_ARCH.X86:
                # if cc == STDCALL:
                return ql.os.x86_stdcall(param_num, params, func, args, kwargs)
                #elif cc == CDECL:
                #    return ql.os.x86_cdecl(param_num, params, func, args, kwargs)
            elif ql.archtype == QL_ARCH.X8664:
                return ql.os.x8664_fastcall(param_num, params, func, args, kwargs)
            elif ql.archtype == QL_ARCH.MIPS:
                return ql.os.mips_o32_call(param_num, params, func, args, kwargs)
            else:
                raise QlErrorArch("Unknown ql.archtype")
        return wrapper
    return decorator


#!/usr/bin/env python3
#
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

import struct
import time
from qiling.os.windows.const import *
from qiling.os.const import *
from qiling.os.windows.fncc import *
from qiling.os.windows.utils import *
from qiling.os.windows.thread import *
from qiling.os.windows.handle import *
from qiling.exception import *
from qiling.const import *


dllname = 'kernel32_dll'

# BOOL SetThreadLocale(
#   LCID Locale
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params_type={'LCID': 'UINT'})
def hook_SetThreadLocale(ql, address, params):
    return 0xC000  # LOCALE_CUSTOM_DEFAULT


# LCID GetThreadLocale();
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GetThreadLocale(ql, address, params):
    return 0xC000  # LOCALE_CUSTOM_DEFAULT


# UINT GetACP(
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GetACP(ql, address, params):
    return OEM_US


# BOOL GetCPInfo(
#   UINT     CodePage,
#   LPCPINFO lpCPInfo
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GetCPInfo(ql, address, params):
    ret = 1
    return ret


# int GetLocaleInfoA(
#   LCID   Locale,
#   LCTYPE LCType,
#   LPSTR  lpLCData,
#   int    cchData
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params_type={'LCID': 'DWORD', 'LCTYPE': 'DWORD'})
def hook_GetLocaleInfoA(ql, address, params):
    locale_value = params["Locale"]
    lctype_value = params["LCType"]
    cchData = params["cchData"]

    local_dict = LOCALE.get(locale_value, None)
    if local_dict is None:
        # raise QlErrorNotImplemented("API not implemented")
        ql.os.last_error = ERROR_INVALID_PARAMETER
        return 0

    lctype = local_dict[lctype_value] + "\x00"

    if cchData != 0:
        lplcdata = params["lpLCData"]
        ql.mem.write(lplcdata, lctype.encode("utf-16le"))
    return len(lctype)


# BOOL IsValidCodePage(
#  UINT CodePage
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_IsValidCodePage(ql, address, params):
    return 1


def _LCMapString(ql, address, params):
    cchDest = params["cchDest"]
    result = (params["lpSrcStr"] + "\x00").encode("utf-16le")
    dst = params["lpDestStr"]
    if cchDest != 0 and dst != 0:
        # TODO maybe do some other check, for now is working
        ql.mem.write(dst, result)
    return len(result)


# int LCMapStringW(
#   LCID    Locale,
#   DWORD   dwMapFlags,
#   LPCWSTR lpSrcStr,
#   int     cchSrc,
#   LPWSTR  lpDestStr,
#   int     cchDest
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_LCMapStringW(ql, address, params):
    return _LCMapString(ql, address, params)


# int LCMapStringA(
#   LCID   Locale,
#   DWORD  dwMapFlags,
#   LPCSTR lpSrcStr,
#   int    cchSrc,
#   LPSTR  lpDestStr,
#   int    cchDest
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_LCMapStringA(ql, address, params):
    return _LCMapString(ql, address, params)


# int LCMapStringEx(
#   LPCWSTR          lpLocaleName,
#   DWORD            dwMapFlags,
#   LPCWSTR          lpSrcStr,
#   int              cchSrc,
#   LPWSTR           lpDestStr,
#   int              cchDest,
#   LPNLSVERSIONINFO lpVersionInformation,
#   LPVOID           lpReserved,
#   LPARAM           sortHandle
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_LCMapStringEx(ql, address, params):
    return _LCMapString(ql, address, params)


# LANGID GetUserDefaultUILanguage();
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GetUserDefaultUILanguage(ql, address, params):
    # TODO find better documentation
    # https://docs.microsoft.com/it-it/windows/win32/intl/language-identifiers
    return ql.os.profile.getint("USER", "language")


# LANGID GetSystemDefaultUILanguage();
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GetSystemDefaultUILanguage(ql, address, params):
    # TODO find better documentation
    # https://docs.microsoft.com/it-it/windows/win32/intl/language-identifiers
    return ql.os.profile.getint("SYSTEM", "language")


# int CompareStringA(
#   LCID   Locale,
#   DWORD  dwCmpFlags,
#   PCNZCH lpString1,
#   int    cchCount1,
#   PCNZCH lpString2,
#   int    cchCount2
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_CompareStringA(ql, address, params):
    st1 = params["lpString1"]
    st2 = params["lpString2"]
    if st1 < st2:
        return CSTR_LESS_THAN
    elif st1 == st2:
        return CSTR_EQUAL
    else:
        return CSTR_GREATER_THAN


#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

from qiling import Qiling


class QlDebugger():
    def __init__(self, ql:Qiling):
        self.ql = ql

    def dbg_start(self):
        pass

    def dbg_run(self, begin_addr=None, end_addr=None):
        self.ql.emu_start(begin=begin_addr, end=end_addr)
    
    def dbg_step(self):
        pass

    def dbg_continue(self):
        pass

    def dbg_set_breakpoint(self):
        pass



#!/usr/bin/env python3
#
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

import struct


from qiling.const import *

PT_DYNAMIC      = 2
DT_NULL 		= 0
DT_NEEDED 		= 1
DT_PLTRELSZ 	= 2
DT_PLTGOT 		= 3
DT_HASH 		= 4
DT_STRTAB 		= 5
DT_SYMTAB 		= 6
DT_RELA 		= 7
DT_RELASZ 		= 8
DT_RELAENT 		= 9
DT_STRSZ 		= 10
DT_SYMENT 		= 11
DT_INIT 		= 12
DT_FINI 		= 13
DT_SONAME 		= 14
DT_RPATH 		= 15
DT_SYMBOLIC 	= 16
DT_REL 			= 17
DT_RELSZ 		= 18
DT_RELENT 		= 19
DT_PLTREL 		= 20
DT_DEBUG 		= 21
DT_TEXTREL 		= 22
DT_JMPREL 		= 23
DT_BIND_NOW 	= 24
DT_INIT_ARRAY 	= 25
DT_FINI_ARRAY 	= 26
DT_INIT_ARRAYSZ = 27
DT_FINI_ARRAYSZ = 28
DT_RUNPATH 		= 29
DT_FLAGS 		= 30
DT_ENCODING 	= 32
DT_GNU_HASH	    = 0x6ffffef5

DT_MIPS_LOCAL_GOTNO = 0x7000000a
DT_MIPS_SYMTABNO = 0x70000011
DT_MIPS_GOTSYM = 0x70000013

class FunctionArgv:
    def __init__(self, ql):
        self.ql = ql

class ARMFunctionArg:
    def __init__(self, ql):
        self.ql = ql
    
    def _get_stack_value(self, idx):
        tmp = self.ql.mem.read(self.ql.reg.sp + idx * self.ql.pointersize, self.ql.pointersize)
        return self.ql.unpack(tmp)
    
    def _set_stack_value(self, idx, value):
        self.ql.mem.write(self.ql.reg.sp + idx * self.ql.pointersize, self.ql.pack(value))

    def __getitem__(self, idx):
        reg_list = ['r0', 'r1', 'r2', 'r3']
        if idx < 4:
            return getattr(self.ql.reg, reg_list[idx])
        else:
            return self._get_stack_value(idx - 4)
    
    def __setitem__(self, idx, value):
        reg_list = ['r0', 'r1', 'r2', 'r3']
        if idx < 4:
            setattr(self.ql.reg, reg_list[idx], value)
        else:
            self._set_stack_value(idx - 4, value)

class MIPS32FunctionArg:
    def __init__(self, ql):
        self.ql = ql

    def _get_stack_value(self, idx):
        tmp = self.ql.mem.read(self.ql.reg.sp + idx * self.ql.pointersize, self.ql.pointersize)
        return self.ql.unpack(tmp)
    
    def _set_stack_value(self, idx, value):
        self.ql.mem.write(self.ql.reg.sp + idx * self.ql.pointersize, self.ql.pack(value))

    def __getitem__(self, idx):
        reg_list = ['a0', 'a1', 'a2', 'a3']
        if idx < 4:
            return getattr(self.ql.reg, reg_list[idx])
        else:
            return self._get_stack_value(idx)
    
    def __setitem__(self, idx, value):
        reg_list = ['a0', 'a1', 'a2', 'a3']
        if idx < 4:
            setattr(self.ql.reg, reg_list[idx], value)
        else:
            self._set_stack_value(idx, value)

class ARM64FunctionArg:
    def __init__(self, ql):
        self.ql = ql

    def _get_stack_value(self, idx):
        tmp = self.ql.mem.read(self.ql.reg.sp + idx * self.ql.pointersize, self.ql.pointersize)
        return self.ql.unpack(tmp)
    
    def _set_stack_value(self, idx, value):
        self.ql.mem.write(self.ql.reg.sp + idx * self.ql.pointersize, self.ql.pack(value))
        
    def __getitem__(self, idx):
        reg_list = ['x0', 'x1', 'x2', 'x3', 'x4', 'x5', 'x6', 'x7']
        if idx < 8:
            return getattr(self.ql.reg, reg_list[idx])
        else:
            return self._get_stack_value(idx - 8 + 1)
    
    def __setitem__(self, idx, value):
        reg_list = ['x0', 'x1', 'x2', 'x3', 'x4', 'x5', 'x6', 'x7']
        if idx < 8:
            setattr(self.ql.reg, reg_list[idx], value)
        else:
            self._set_stack_value(idx - 8 + 1, value)

class X86FunctionArg:
    def __init__(self, ql):
        self.ql = ql

    def _get_stack_value(self, idx):
        tmp = self.ql.mem.read(self.ql.reg.esp + idx * self.ql.pointersize, self.ql.pointersize)
        return self.ql.unpack(tmp)
    
    def _set_stack_value(self, idx, value):
        self.ql.mem.write(self.ql.reg.esp + idx * self.ql.pointersize, self.ql.pack(value))

    def __getitem__(self, idx):
        return self._get_stack_value(idx + 1)
    
    def __setitem__(self, idx, value):
        self._set_stack_value(idx + 1, value)

class X64FunctionArg:
    def __init__(self, ql):
        self.ql = ql

    def _get_stack_value(self, idx):
        tmp = self.ql.mem.read(self.ql.reg.rsp + idx * self.ql.pointersize, self.ql.pointersize)
        return self.ql.unpack(tmp)
    
    def _set_stack_value(self, idx, value):
        self.ql.mem.write(self.ql.reg.rsp + idx * self.ql.pointersize, self.ql.pack(value))

    def __getitem__(self, idx):
        reg_list = ['rdi', 'rsi', 'rdx', 'rcx', 'r8', 'r9']
        if idx < 6:
            return getattr(self.ql.reg, reg_list[idx])
        else:
            return self._get_stack_value(idx - 6 + 1)
    
    def __setitem__(self, idx, value):
        reg_list = ['rdi', 'rsi', 'rdx', 'rcx', 'r8', 'r9']
        if idx < 6:
            setattr(self.ql.reg, reg_list[idx], value)
        else:
            self._set_stack_value(idx - 6 + 1, value)

class HookFunc:
    def __init__(self, ql, fn):
        self.fucname = fn
        self.hook_data_ptr = None
        self.hook_onenter = []
        self.hook_onexit = []
        self.hook = []
        self.ql = ql
        self.ret_pc = None
        self.exit_addr = None

    def add_hook(self, cb, intercept, userdata):
        if intercept == QL_INTERCEPT.ENTER:
            self.hook_onenter.append((cb, userdata))
        elif intercept == QL_INTERCEPT.EXIT:
            self.hook_onexit.append((cb, userdata))
        else:
            self.hook.append((cb, userdata))

    def get_ret_pc(self):
        # ARM
        if self.ql.archtype== QL_ARCH.ARM:
            return self.ql.reg.lr

        # MIPS32
        elif self.ql.archtype== QL_ARCH.MIPS:
            return self.ql.reg.ra

        # ARM64
        elif self.ql.archtype== QL_ARCH.ARM64:
            return self.ql.reg.x30

        # X86
        elif  self.ql.archtype== QL_ARCH.X86:
            return self.ql.unpack(self.ql.mem.read(self.ql.reg.esp, self.ql.pointersize))

        # X8664
        elif  self.ql.archtype== QL_ARCH.X8664:
            return self.ql.unpack(self.ql.mem.read(self.ql.reg.rsp, self.ql.pointersize))
        else:
            raise

    def context_fixup(self):
        # ARM
        if self.ql.archtype== QL_ARCH.ARM:
            pass

        # MIPS32
        elif self.ql.archtype== QL_ARCH.MIPS:
            pass

        # ARM64
        elif self.ql.archtype== QL_ARCH.ARM64:
            pass

        # X86
        elif  self.ql.archtype== QL_ARCH.X86:
            self.ql.reg.esp = self.ql.reg.esp + self.ql.pointersize

        # X8664
        elif  self.ql.archtype== QL_ARCH.X8664:
            self.ql.reg.rsp = self.ql.reg.rsp + self.ql.pointersize
        else:
            raise

    def set_ret(self, addr):
        # ARM
        if self.ql.archtype== QL_ARCH.ARM:
            self.ql.reg.lr = addr

        # MIPS32
        elif self.ql.archtype== QL_ARCH.MIPS:
            self.ql.reg.ra = addr

        # ARM64
        elif self.ql.archtype== QL_ARCH.ARM64:
            self.ql.mem.write(self.ql.reg.sp, self.ql.pack(addr))

        # X86
        elif  self.ql.archtype== QL_ARCH.X86:
            self.ql.mem.write(self.ql.reg.esp, self.ql.pack(addr))

        # X8664
        elif  self.ql.archtype== QL_ARCH.X8664:
            self.ql.mem.write(self.ql.reg.rsp, self.ql.pack(addr))
        else:
            raise

    def call_enter(self):
        # if self.ql.archtype == QL_ARCH.ARM or self.ql.archtype == QL_ARCH.ARM64:
        #     self.ql.reg.arch_pc = self.ql.reg.arch_pc + 4

        next_pc = self.ql.unpack(self.ql.mem.read(self.hook_data_ptr, self.ql.pointersize))
        self.ret_pc = self.get_ret_pc()
        onenter_cb = None
        onenter_userdata = None

        if len(self.hook_onenter) != 0:
            onenter_cb, onenter_userdata = self.hook_onenter[0]
        
        cb = None
        userdata = None
        if len(self.hook) != 0:
            cb, userdata = self.hook[0]
        
        if onenter_cb == None:
            ret = 0
        else:
            if onenter_userdata == None:
                ret = onenter_cb(self.ql)
            else:
                ret = onenter_cb(self.ql, onenter_userdata)

        if isinstance(ret, int) == False or ret & QL_CALL_BLOCK == 0:
            if cb != None:
                if userdata == None:
                    cb(self.ql)
                else:
                    cb(self.ql, userdata)
            else:
                self.set_ret(self.exit_addr)
                self.ql.reg.arch_pc = next_pc

        else:
            self.context_fixup()
    
    def ret(self):
        # ARM
        if self.ql.archtype== QL_ARCH.ARM:
            self.ql.reg.arch_pc = self.ret_pc

        # MIPS32
        elif self.ql.archtype== QL_ARCH.MIPS:
            self.ql.reg.arch_pc = self.ret_pc

        # ARM64
        elif self.ql.archtype== QL_ARCH.ARM64:
            self.ql.reg.arch_pc = self.ret_pc

        # X86
        elif  self.ql.archtype== QL_ARCH.X86:
            self.ql.reg.arch_pc = self.ret_pc

        # X8664
        elif  self.ql.archtype== QL_ARCH.X8664:
            self.ql.reg.arch_pc = self.ret_pc
        else:
            raise

    def call_exit(self):
        # if self.ql.archtype == QL_ARCH.ARM or self.ql.archtype == QL_ARCH.ARM64:
        #     self.ql.reg.arch_pc = self.ql.reg.arch_pc + 4
        onexit_cb = None
        onexit_userdata = None

        if len(self.hook_onexit) != 0:
            onexit_cb, onexit_userdata = self.hook_onexit[0]

        if onexit_cb != None:
            if onexit_userdata == None:
                onexit_cb(self.ql)
            else:
                onexit_cb(self.ql, onexit_userdata)

        self.ret()


class HookFuncRel(HookFunc):
    def __init__(self, ql, fucname, r, load_base):
        super(HookFuncRel, self).__init__(ql, fucname)
        self.rel = r
        self.idx = None
        self.load_base = load_base
        self.hook_fuc_ptr = None
        self.ori_offest = None
        self.ori_data = None
    
    def _hook_fuc_enter(self, ql):
        self.call_enter()
    
    def _hook_fuc_exit(self, ql):
        self.call_exit()
    
    def enable(self):
        if self.rel == None or self.hook_fuc_ptr == None or self.hook_data_ptr == None:
            raise
        
        self.exit_addr = self.hook_fuc_ptr + 8

        self.ql.hook_address(self._hook_fuc_enter, self.hook_fuc_ptr)
        self.ql.hook_address(self._hook_fuc_exit, self.hook_fuc_ptr + 8)
        
        self.ori_offest = self.rel.r_offset
        self.rel.r_offset = self.hook_data_ptr - self.load_base
        self.ori_data = self.ql.mem.read(self.ori_offest + self.load_base, self.ql.pointersize)        
        
        self.ql.mem.write(self.rel.ptr, self.rel.pack())
        self.ql.mem.write(self.ori_offest + self.load_base, self.ql.pack(self.hook_fuc_ptr))
        self.ql.mem.write(self.hook_data_ptr, bytes(self.ori_data))

class HookFuncMips(HookFunc):
    def __init__(self, ql, fucname, got, gotidx, load_base):
        super(HookFuncMips, self).__init__(ql, fucname)
        self.load_base = load_base
        self.got = got
        self.gotidx = gotidx
    
    def _hook_fuc_enter(self, ql):
        self.ql.reg.t9 = self.ql.unpack(self.ql.mem.read(self.hook_data_ptr, self.ql.pointersize))
        self.call_enter()
    
    def _hook_fuc_exit(self, ql):
        self.call_exit()

        tmp = self.ql.unpack(self.ql.mem.read(self.got + self.load_base + self.gotidx * self.ql.pointersize, self.ql.pointersize))
        if tmp != self.hook_fuc_ptr:
            self.ql.mem.write(self.got + self.load_base + self.gotidx * self.ql.pointersize, self.ql.pack(self.hook_fuc_ptr))
            self.ql.mem.write(self.hook_data_ptr, self.ql.pack(tmp))
    
    def _hook_got(self):
        self.ori_data = self.ql.mem.read(self.got + self.load_base + self.gotidx * self.ql.pointersize, self.ql.pointersize) 

        self.ql.mem.write(self.got + self.load_base + self.gotidx * self.ql.pointersize, self.ql.pack(self.hook_fuc_ptr))
        self.ql.mem.write(self.hook_data_ptr, bytes(self.ori_data))
    
    def enable(self):
        if self.got == None or self.hook_fuc_ptr == None or self.hook_data_ptr == None:
            raise

        self.ql.os.register_function_after_load(self._hook_got)
        
        self.exit_addr = self.hook_fuc_ptr + 8

        self.ql.hook_address(self._hook_fuc_enter, self.hook_fuc_ptr)
        self.ql.hook_address(self._hook_fuc_exit, self.hook_fuc_ptr + 8)


class ELF_Phdr:
    def __init__(self, p_type, p_offset, p_vaddr, p_paddr, p_filesz, p_memsz, p_flags, p_align):
        self.p_type = p_type
        self.p_offset = p_offset
        self.p_vaddr = p_vaddr
        self.p_paddr = p_paddr
        self.p_filesz = p_filesz
        self.p_memsz = p_memsz
        self.p_flags = p_flags
        self.p_align = p_align

class ELF32_Phdr(ELF_Phdr):
    Phdr_SIZE = 4 * 8
    def __init__(self, buf, endian = 0):
        if len(buf) != self.Phdr_SIZE:
            raise

        fmt = '<IIIIIIII' if endian == 0 else '>IIIIIIII'

        p_type, p_offset, p_vaddr, p_paddr, p_filesz, p_memsz, p_flags, p_align = struct.unpack(fmt, buf)
        super(ELF32_Phdr, self).__init__(p_type, p_offset, p_vaddr, p_paddr, p_filesz, p_memsz, p_flags, p_align)

class ELF64_Phdr(ELF_Phdr):
    Phdr_SIZE = 8 * 7
    def __init__(self, buf, endian = 0):
        if len(buf) != self.Phdr_SIZE:
            raise
        
        fmt = '<IIQQQQQQ' if endian == 0 else '>IIQQQQQQ'

        p_type, p_flags, p_offset, p_vaddr, p_paddr, p_filesz, p_memsz, p_align = struct.unpack(fmt, buf)
        super(ELF64_Phdr, self).__init__(p_type, p_offset, p_vaddr, p_paddr, p_filesz, p_memsz, p_flags, p_align)
        
class ELF_Dyn:
    def __init__(self, d_tag, d_un):
        self.d_tag = d_tag
        self.d_un = d_un

class ELF32_Dyn(ELF_Dyn):
    Dyn_SIZE = 4 * 2
    def __init__(self, buf, endian = 0):
        if len(buf) != self.Dyn_SIZE:
            raise
        
        fmt = '<iI' if endian == 0 else '>iI'

        d_tag, d_un = struct.unpack(fmt, buf)
        super(ELF32_Dyn, self).__init__(d_tag, d_un)

class ELF64_Dyn(ELF_Dyn):
    Dyn_SIZE = 8 * 2
    def __init__(self, buf, endian = 0):
        if len(buf) != self.Dyn_SIZE:
            raise
        
        fmt = '<qQ' if endian == 0 else '>qQ'

        d_tag, d_un = struct.unpack(fmt, buf)
        super(ELF64_Dyn, self).__init__(d_tag, d_un)

class ELF_Rel:
    def __init__(self, r_offset, r_info):
        self.r_offset = r_offset
        self.r_info = r_info

class ELF32_Rel(ELF_Rel):
    Rel_SIZE = 4 * 2
    def __init__(self, buf, endian = 0, ptr = None):
        if len(buf) != self.Rel_SIZE:
            raise
        
        self.ptr = ptr
        self.fmt = '<II' if endian == 0 else '>II'

        r_offset, r_info = struct.unpack(self.fmt, buf)
        super(ELF32_Rel, self).__init__(r_offset, r_info)

    @property
    def r_type(self):
        return self.r_info & 0xff

    @property
    def r_sym(self):
        return self.r_info >> 8
    
    def pack(self):
        return struct.pack(self.fmt, self.r_offset, self.r_info)

class ELF64_Rel(ELF_Rel):
    Rel_SIZE = 8 * 2
    def __init__(self, buf, endian = 0, ptr = None):
        if len(buf) != self.Rel_SIZE:
            raise
        
        self.ptr = ptr
        self.fmt = '<QQ' if endian == 0 else '>QQ'

        r_offset, r_info = struct.unpack(self.fmt, buf)
        super(ELF64_Rel, self).__init__(r_offset, r_info)

    @property
    def r_type(self):
        return self.r_info & 0xffffffff
        
    @property
    def r_sym(self):
        return self.r_info >> 32
    
    def pack(self):
        return struct.pack(self.fmt, self.r_offset, self.r_info)


class ELF_Rela:
    def __init__(self, r_offset, r_info, r_addend):
        self.r_offset = r_offset
        self.r_info = r_info
        self.r_addend = r_addend

class ELF32_Rela(ELF_Rela):
    Rela_SIZE = 4 * 3
    def __init__(self, buf, endian = 0, ptr = None):
        if len(buf) != self.Rela_SIZE:
            raise
        
        self.ptr = ptr
        self.fmt = '<IIi' if endian == 0 else '>IIi'

        r_offset, r_info, r_addend = struct.unpack(self.fmt, buf)
        super(ELF32_Rela, self).__init__(r_offset, r_info, r_addend)

    @property
    def r_type(self):
        return self.r_info & 0xff

    @property
    def r_sym(self):
        return self.r_info >> 8
    
    def pack(self):
        return struct.pack(self.fmt, self.r_offset, self.r_info, self.r_addend)

class ELF64_Rela(ELF_Rela):
    Rela_SIZE = 8 * 3
    def __init__(self, buf, endian = 0, ptr = None):
        if len(buf) != self.Rela_SIZE:
            raise
        
        self.ptr = ptr
        self.fmt = '<QQq' if endian == 0 else '>QQq'

        r_offset, r_info, r_addend = struct.unpack(self.fmt, buf)
        super(ELF64_Rela, self).__init__(r_offset, r_info, r_addend)

    @property
    def r_type(self):
        return self.r_info & 0xffffffff
        
    @property
    def r_sym(self):
        return self.r_info >> 32

    def pack(self):
        return struct.pack(self.fmt, self.r_offset, self.r_info, self.r_addend)

class ELF_Sym:
    def __init__(self, st_name ,st_value ,st_size ,st_info ,st_other ,st_shndx):
        self.st_name = st_name
        self.st_value = st_value
        self.st_size = st_size
        self.st_info = st_info
        self.st_other = st_other
        self.st_shndx = st_shndx

class ELF32_Sym(ELF_Sym):
    Sym_SIZE = 4 * 4
    def __init__(self, buf, endian = 0):
        if len(buf) != self.Sym_SIZE:
            raise
        
        self.fmt = '<IIIBBH' if endian == 0 else '>IIIBBH'

        st_name ,st_value ,st_size ,st_info ,st_other ,st_shndx = struct.unpack(self.fmt, buf)
        super(ELF32_Sym, self).__init__(st_name ,st_value ,st_size ,st_info ,st_other ,st_shndx)
    
    def pack(self):
        struct.pack(self.fmt ,self.st_name ,self.st_value ,self.st_size ,self.st_info ,self.st_other ,self.st_shndx)

class ELF64_Sym(ELF_Sym):
    Sym_SIZE = 8 * 3
    def __init__(self, buf, endian = 0):
        if len(buf) != self.Sym_SIZE:
            raise
        
        self.fmt = '<IBBHQQ' if endian == 0 else '>IBBHQQ'

        st_name ,st_info ,st_other ,st_shndx ,st_value ,st_size = struct.unpack(self.fmt, buf)
        super(ELF64_Sym, self).__init__(st_name ,st_value ,st_size ,st_info ,st_other ,st_shndx)
    
    def pack(self):
        struct.pack(self.fmt ,self.st_name ,self.st_info ,self.st_other ,self.st_shndx ,self.st_value ,self.st_size)

class ELF_Symtab:
    def __init__(self, ql, symtab, endian = 0):
        self.ql = ql
        self.symtab = symtab
        self.endian = endian

        self.symclass = ELF32_Sym if self.ql.archbit == 32 else ELF64_Sym
    
    def __getitem__(self, idx):
        buf = self.ql.mem.read(self.symtab + idx * self.symclass.Sym_SIZE, self.symclass.Sym_SIZE)
        return self.symclass(buf, self.endian)

    def __setitem__(self, idx, sym):
        self.ql.mem.write(self.symtab + idx * self.symclass.Sym_SIZE, sym.pack())

class ELF_Strtab:
    def __init__(self, strtab):
        self.strtab = bytes(strtab)
    
    def __getitem__(self, idx):
        return self.strtab[idx: self.strtab.index(b'\x00', idx)]

class FunctionHook:
    def __init__(self, ql, phoff, phnum, phentsize, load_base, hook_mem):
        self.ql = ql
        self.hook_mem = hook_mem
        self.phoff = phoff
        self.phnum = phnum
        self.phentsize = phentsize
        self.load_base = load_base
        self.add_function_hook = self.add_function_hook_default 

        self.dynamic = None

        self.hash_nbucket = None
        self.hash_nchain = None
        self.hash_bucket = None
        self.hash_chain = None

        self.gnu_nbucket = None
        self.gnu_symbias = None
        self.gnu_maskwords = None
        self.gnu_shift2 = None
        self.gnu_bloom_filter = None
        self.gnu_bucket = None
        self.gnu_chain = None

        self.strtab = None
        self.strtab_size = None

        self.symtab = None
        self.syment = ELF32_Sym.Sym_SIZE if ql.archbit == 32 else ELF64_Sym.Sym_SIZE

        self.plt_rel_size = None
        self.plt_rel = None
        self.plt_rel_type = DT_REL if ql.archbit == 32 else DT_RELA

        self.rela = None
        self.rela_size = None
        self.relaent = ELF32_Rela.Rela_SIZE if ql.archbit == 32 else ELF64_Rela.Rela_SIZE

        self.rel = None
        self.rel_size = None
        self.relent = ELF32_Rel.Rel_SIZE if ql.archbit == 32 else ELF64_Rel.Rel_SIZE

        self.plt_got = None
        self.mips_local_gotno = None
        self.mips_symtabno = None
        self.mips_gotsym = None

        self.rel_list = []
        self.endian = 0 if ql.archendian == QL_ENDIAN.EL else 1

        # ARM
        if self.ql.archtype in [QL_ARCH.ARM, QL_ARCH.ARM_THUMB]:
            self.GLOB_DAT = 21
            self.JMP_SLOT = 22
            # orr r1, r1, r1
            ins = b'\x01\x10\x81\xe1'
            self.add_function_hook = self.add_function_hook_relocation

        # MIPS32
        elif self.ql.archtype== QL_ARCH.MIPS:
            self.GLOB_DAT = 21
            self.JMP_SLOT = 22
            # add $t9, $t9, $zero
            ins = b' \xc8 \x03'
            self.add_function_hook = self.add_function_hook_mips

        # ARM64
        elif self.ql.archtype== QL_ARCH.ARM64:
            self.GLOB_DAT = 1025
            self.JMP_SLOT = 1026
            # orr x1,x1,x1
            ins = b'\x21\x00\x01\xaa'
            self.add_function_hook = self.add_function_hook_relocation

        # X86
        elif  self.ql.archtype== QL_ARCH.X86:
            self.GLOB_DAT = 6
            self.JMP_SLOT = 7
            # nop
            ins = b'\x90'
            self.add_function_hook = self.add_function_hook_relocation

        # X8664
        elif  self.ql.archtype== QL_ARCH.X8664:
            self.GLOB_DAT = 6
            self.JMP_SLOT = 7
            # nop
            ins = b'\x90'
            self.add_function_hook = self.add_function_hook_relocation

        self._parse()
        if self.rel != None:
            self.show_relocation(self.rel)

        if self.rela != None:
            self.rel_list += self.rela
            self.show_relocation(self.rela)

        if self.plt_rel != None:
            self.rel_list += self.plt_rel
            self.show_relocation(self.plt_rel)
        
        if self.ql.archtype == QL_ARCH.MIPS and self.plt_got != None and self.mips_gotsym != None and self.mips_local_gotno != None and self.mips_symtabno != None:
            self.show_dynsym_name(self.mips_gotsym, self.mips_symtabno)

        self.ql.mem.map(hook_mem, 0x2000, perms=7, info="[hook_mem]")
        self.ql.mem.write(hook_mem, ins * (0x1000 // len(ins)))

        self.free_list = [_ for _ in range(0, 0x1000, 0x10)]
        self.use_list = {}
        self.hook_list = {}

        self.hook_int = False

    def parse_program_header32(self):
        # typedef struct elf32_phdr{
        # Elf32_Word	p_type;
        # Elf32_Off	p_offset;
        # Elf32_Addr	p_vaddr;
        # Elf32_Addr	p_paddr;
        # Elf32_Word	p_filesz;
        # Elf32_Word	p_memsz;
        # Elf32_Word	p_flags;
        # Elf32_Word	p_align;
        # } Elf32_Phdr;

        # /* 32-bit ELF base types. */
        # typedef uint32_t Elf32_Addr;
        # typedef uint16_t Elf32_Half;
        # typedef uint32_t Elf32_Off;
        # typedef int32_t  Elf32_Sword;
        # typedef uint32_t Elf32_Word;

        Psize = int(self.phentsize)
        Pnum = int(self.phnum)
        Pdata = self.ql.mem.read(self.phoff, Pnum * Psize)

        for i in range(Pnum):
            buf = Pdata[i * ELF32_Phdr.Phdr_SIZE : (i + 1) * ELF32_Phdr.Phdr_SIZE]
            P = ELF32_Phdr(buf, self.endian)
            yield P
        return

    def parse_program_header64(self):
        # typedef struct elf64_phdr {
        # Elf64_Word p_type;
        # Elf64_Word p_flags;
        # Elf64_Off p_offset;		/* Segment file offset */
        # Elf64_Addr p_vaddr;		/* Segment virtual address */
        # Elf64_Addr p_paddr;		/* Segment physical address */
        # Elf64_Xword p_filesz;		/* Segment size in file */
        # Elf64_Xword p_memsz;		/* Segment size in memory */
        # Elf64_Xword p_align;		/* Segment alignment, file & memory */
        # } Elf64_Phdr;

        # /* 64-bit ELF base types. */
        # typedef uint64_t Elf64_Addr;
        # typedef uint16_t Elf64_Half;
        # typedef int16_t	 Elf64_SHalf;
        # typedef uint64_t Elf64_Off;
        # typedef int32_t	 Elf64_Sword;
        # typedef uint32_t Elf64_Word;
        # typedef uint64_t Elf64_Xword;
        # typedef int64_t  Elf64_Sxword;

        Psize = int(self.phentsize)
        Pnum = int(self.phnum)
        Pdata = self.ql.mem.read(self.phoff, Pnum * Psize)

        for i in range(Pnum):
            buf = Pdata[i * ELF64_Phdr.Phdr_SIZE : (i + 1) * ELF64_Phdr.Phdr_SIZE]
            P = ELF64_Phdr(buf, self.endian)
            yield P
        return

    def parse_program_header(self):
        if self.ql.archbit == 64:
            return self.parse_program_header64()
        elif self.ql.archbit == 32:
            return self.parse_program_header32()

    def parse_dynamic64(self):
        # typedef struct
        # {
        # Elf64_Sxword	d_tag;			/* Dynamic entry type */
        # union
        #     {
        #     Elf64_Xword d_val;		/* Integer value */
        #     Elf64_Addr d_ptr;			/* Address value */
        #     } d_un;
        # } Elf64_Dyn;

        # /* 64-bit ELF base types. */
        # typedef uint64_t Elf64_Addr;
        # typedef uint16_t Elf64_Half;
        # typedef int16_t	 Elf64_SHalf;
        # typedef uint64_t Elf64_Off;
        # typedef int32_t	 Elf64_Sword;
        # typedef uint32_t Elf64_Word;
        # typedef uint64_t Elf64_Xword;
        # typedef int64_t  Elf64_Sxword;

        Dsize = ELF64_Dyn.Dyn_SIZE
        idx = 0

        while True:
            buf = self.ql.mem.read(self.dynamic + idx * Dsize, Dsize)
            D = ELF64_Dyn(buf, self.endian)
            yield D
            idx += 1
            if D.d_tag == DT_NULL:
                break
        return
    
    def parse_dynamic32(self):
        # typedef struct
        # {
        # Elf32_Sword	d_tag;			/* Dynamic entry type */
        # union
        #     {
        #     Elf32_Word d_val;			/* Integer value */
        #     Elf32_Addr d_ptr;			/* Address value */
        #     } d_un;
        # } Elf32_Dyn;

        # /* 32-bit ELF base types. */
        # typedef uint32_t Elf32_Addr;
        # typedef uint16_t Elf32_Half;
        # typedef uint32_t Elf32_Off;
        # typedef int32_t  Elf32_Sword;
        # typedef uint32_t Elf32_Word;
        Dsize = ELF32_Dyn.Dyn_SIZE
        idx = 0

        while True:
            buf = self.ql.mem.read(self.dynamic + idx * Dsize, Dsize)
            D = ELF32_Dyn(buf, self.endian)
            yield D
            idx += 1
            if D.d_tag == DT_NULL:
                break
        return

    def parse_dynamic(self):
        if self.ql.archbit == 64:
            return self.parse_dynamic64()
        elif self.ql.archbit == 32:
            return self.parse_dynamic32()

    def _parse(self):
        for p in self.parse_program_header():
            if p.p_type == PT_DYNAMIC:
                self.dynamic = p.p_vaddr + self.load_base
                break
        
        if self.dynamic == None:
            return

        for d in self.parse_dynamic():
            if d.d_tag == DT_NULL:
                break
            elif d.d_tag == DT_HASH:
                # self.hash_nbucket = self.ql.unpack(self.ql.mem.read(self.load_base + d.d_un, self.ql.pointersize))
                # self.hash_nchain = self.ql.unpack(self.ql.mem.read(self.load_base + d.d_un + self.ql.pointersize, self.ql.pointersize))
                # self.hash_bucket = self.ql.mem.read(self.load_base + d.d_un + self.ql.pointersize * 2, self.ql.pointersize * self.hash_nbucket)
                # self.hash_chain = self.ql.unpack(self.ql.mem.read(self.load_base + d.d_un + self.ql.pointersize * 2 + self.ql.pointersize * self.hash_nbucket, self.ql.pointersize))
                pass
            elif d.d_tag == DT_GNU_HASH:
                # self.gnu_nbucket = self.ql.unpack(self.ql.mem.read(self.load_base + d.d_un, self.ql.pointersize))
                # self.gnu_symbias = self.ql.unpack(self.ql.mem.read(self.load_base + d.d_un + self.ql.pointersize, self.ql.pointersize))))
                # self.gnu_maskwords = self.ql.unpack(self.ql.mem.read(self.load_base + d.d_un + self.ql.pointersize * 2, self.ql.pointersize))
                # self.gnu_shift2 = self.ql.unpack(self.ql.mem.read(self.load_base + d.d_un + self.ql.pointersize * 3, self.ql.pointersize))
                # self.gnu_bloom_filter = self.ql.mem.read(self.load_base + d.d_un + self.ql.pointersize * 4, self.gnu_maskwords)
                # self.gnu_bucket = self.ql.mem.read(self.load_base + d.d_un + self.ql.pointersize * 4 + self.gnu_maskwords, self.ql.pointersize * self.gnu_nbucket)
                # self.gnu_chain = self.load_base + d.d_un + self.ql.pointersize * 4 + self.gnu_maskwords + self.ql.pointersize * self.gnu_nbucket - self.ql.pointersize * self.gnu_symbias
                pass

            elif d.d_tag == DT_STRTAB:
                self.strtab = d.d_un + self.load_base
            elif d.d_tag == DT_STRSZ:
                self.strtab_size = d.d_un

            elif d.d_tag == DT_SYMTAB:
                self.symtab = d.d_un + self.load_base
            elif d.d_tag == DT_SYMENT:
                if d.d_un != self.syment:
                    raise

            elif d.d_tag == DT_PLTREL:
                if d.d_un != self.plt_rel_type:
                    raise
            elif d.d_tag == DT_PLTRELSZ:
                self.plt_rel_size = d.d_un
            elif d.d_tag == DT_JMPREL:
                self.plt_rel = d.d_un + self.load_base

            elif d.d_tag == DT_RELA:
                self.rela = d.d_un + self.load_base
            elif d.d_tag == DT_RELASZ:
                self.rela_size = d.d_un
            elif d.d_tag == DT_RELAENT:
                if self.relaent != d.d_un:
                    raise

            elif d.d_tag == DT_REL:
                self.rel = d.d_un + self.load_base
            elif d.d_tag == DT_RELSZ:
                self.rel_size = d.d_un
            elif d.d_tag == DT_RELENT:
                if self.relent != d.d_un:
                    raise

            elif d.d_tag == DT_PLTGOT:
                self.plt_got = d.d_un

            elif d.d_tag == DT_MIPS_LOCAL_GOTNO:
                self.mips_local_gotno = d.d_un
            elif d.d_tag == DT_MIPS_SYMTABNO:
                self.mips_symtabno = d.d_un
            elif d.d_tag == DT_MIPS_GOTSYM:
                self.mips_gotsym = d.d_un
            
            elif d.d_tag == DT_NEEDED:
                pass
            else:
                pass
            
        if self.strtab != None and self.strtab_size != None:
            self.strtab = ELF_Strtab(self.ql.mem.read(self.strtab, self.strtab_size))
        
        if self.rela != None and self.rela_size != None:
            rela_buf = self.ql.mem.read(self.rela, self.rela_size)
            rela_ptr = self.rela
            if self.ql.archbit == 32:
                self.rela = [ELF32_Rela(rela_buf[_ * self.relaent : (_ + 1) * self.relaent], self.endian, rela_ptr + _ * self.relaent) for _ in range(self.rela_size // self.relaent)]
            elif self.ql.archbit == 64:
                self.rela = [ELF64_Rela(rela_buf[_ * self.relaent : (_ + 1) * self.relaent], self.endian, rela_ptr + _ * self.relaent) for _ in range(self.rela_size // self.relaent)]
        
        if self.rel != None and self.rel_size != None:
            rel_buf = self.ql.mem.read(self.rel, self.rel_size)
            rel_ptr = self.rel
            if self.ql.archbit == 32:
                self.rel = [ELF32_Rel(rel_buf[_ * self.relent : (_ + 1) * self.relent], self.endian, rel_ptr + _ * self.relent) for _ in range(self.rel_size // self.relent)]
            elif self.ql.archbit == 64:
                self.rel = [ELF64_Rel(rel_buf[_ * self.relent : (_ + 1) * self.relent], self.endian, rel_ptr + _ * self.relent) for _ in range(self.rel_size // self.relent)]

        if self.plt_rel != None and self.plt_rel_size != None:
            plt_rel_buf = self.ql.mem.read(self.plt_rel, self.plt_rel_size)
            plt_rel_ptr = self.plt_rel
            if self.plt_rel_type == DT_REL:
                if self.ql.archbit == 32:
                    self.plt_rel = [ELF32_Rel(plt_rel_buf[_ * self.relent : (_ + 1) * self.relent], self.endian, plt_rel_ptr + _ * self.relent) for _ in range(self.plt_rel_size // self.relent)]
                elif self.ql.archbit == 64:
                    self.plt_rel = [ELF64_Rel(plt_rel_buf[_ * self.relent : (_ + 1) * self.relent], self.endian, plt_rel_ptr + _ * self.relent) for _ in range(self.plt_rel_size // self.relent)]
            else:
                if self.ql.archbit == 32:
                    self.plt_rel = [ELF32_Rela(plt_rel_buf[_ * self.relaent : (_ + 1) * self.relaent], self.endian, plt_rel_ptr + _ * self.relaent) for _ in range(self.plt_rel_size // self.relaent)]
                elif self.ql.archbit == 64:
                    self.plt_rel = [ELF64_Rela(plt_rel_buf[_ * self.relaent : (_ + 1) * self.relaent], self.endian, plt_rel_ptr + _ * self.relaent) for _ in range(self.plt_rel_size // self.relaent)]
        
        if self.symtab != None:
            self.symtab = ELF_Symtab(self.ql, self.symtab, self.endian)
        
    
    def show_relocation(self, rel):
        for r in rel:
            if (r.r_type == self.JMP_SLOT or r.r_type == self.GLOB_DAT) and r.r_sym != 0:
                rel_name = self.strtab[self.symtab[r.r_sym].st_name]
                self.ql.log.debug('rel name ' + str(rel_name))
    
    def show_dynsym_name(self, s, e):
        for symidx in range(s, e):
            rel_name = self.strtab[self.symtab[symidx].st_name]
            self.ql.log.debug('dynsym name ' + str(rel_name))

    def _hook_int(self, ql, intno):
        idx = (self.ql.reg.arch_pc - self.hook_mem) // 0x10

        if idx not in self.use_list.keys():
            raise

        self.use_list[idx].call()

    def _hook_function(self, fn, r, cb, pos, userdata):
        if fn in self.hook_list.keys():
            self.hook_list[fn].add_hook(cb, pos, userdata)
            return

        hf = HookFuncRel(self.ql, fn, r, self.load_base)
        hf.add_hook(cb, pos, userdata)

        if len(self.free_list) == 0:
            raise
        
        hf.idx = self.free_list[0]
        del self.free_list[0]

        hf.hook_fuc_ptr = hf.idx * 0x10 + self.hook_mem
        hf.hook_data_ptr = hf.idx * 0x10 + self.hook_mem + 0x1000

        self.use_list[hf.idx] = hf
        self.hook_list[fn] = hf

        hf.enable()

        # if self.hook_int == False:
        #     if self.ql.archtype == QL_ARCH.X86 or self.ql.archtype == QL_ARCH.X8664:
        #         self.ql.hook_intno(self._hook_int, 0xa0)
        #     elif self.ql.archtype == QL_ARCH.ARM or self.ql.archtype == QL_ARCH.ARM64:
        #         self.ql.hook_intno(self._hook_int, 7)
        #     self.hook_int = True


    def add_function_hook_relocation(self, funcname, cb, pos, userdata = None):
        if type(funcname) != str:
            raise
        
        for r in self.rel_list:
            if (r.r_type == self.JMP_SLOT or r.r_type == self.GLOB_DAT) and r.r_sym != 0:
                tmp_name = self.strtab[self.symtab[r.r_sym].st_name]
                if tmp_name == funcname.encode():
                    self._hook_function(tmp_name, r, cb, pos, userdata)
    
    def add_function_hook_default(self, funcname, cb, userdata = None):
        pass
    
    def add_function_hook_mips(self, funcname, cb, pos, userdata = None):
        self.add_function_hook_relocation(funcname, cb, userdata)

        for symidx in range(self.mips_gotsym, self.mips_symtabno):
            tmp_name = self.strtab[self.symtab[symidx].st_name]
            if tmp_name == funcname.encode():
                fn = tmp_name
                if fn in self.hook_list.keys():
                    self.hook_list[fn].add_hook(cb, pos, userdata)
                    return

                hf = HookFuncMips(self.ql, fn, self.plt_got, symidx - self.mips_gotsym + self.mips_local_gotno, self.load_base)
                hf.add_hook(cb, pos, userdata)

                if len(self.free_list) == 0:
                    raise
                
                hf.idx = self.free_list[0]
                del self.free_list[0]

                hf.hook_fuc_ptr = hf.idx * 0x10 + self.hook_mem
                hf.hook_data_ptr = hf.idx * 0x10 + self.hook_mem + 0x1000

                self.use_list[hf.idx] = hf
                self.hook_list[fn] = hf

                hf.enable()

                # if self.hook_int == False:
                #     self.ql.hook_intno(self._hook_int, 0xa0)
                #     self.hook_int = True

    def _load_import(self):
        pass


#!/usr/bin/env python3
#
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

import struct
import time
from qiling.os.windows.const import *
from qiling.os.const import *
from qiling.os.windows.fncc import *
from qiling.os.windows.utils import *
from qiling.os.windows.thread import *
from qiling.os.windows.handle import *
from qiling.exception import *

dllname = 'oleaut32_dll'

# TODO this file is VERY experimental.

# BSTR SysAllocStringLen(
#   const OLECHAR *strIn,
#   UINT          ui
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_SysAllocStringLen(ql, address, params):
    addr = ql.os.heap.alloc(params["ui"] + 1)
    if params["strIn"] != 0:
        ql.mem.write(addr, params["strIn"][:params["ui"]].encode("utf-16le"))
    return addr


# void SysFreeString(
#   BSTR bstrString
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_SysFreeString(ql, address, params):
    addr = params["strIn"]
    if addr != 0:
        ql.os.heap.free(addr)
        content = ql.os.read_wstring(addr)

        params["strIn"] = content
    return 0


# UINT SysStringLen(
#   BSTR pbstr
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_SysStringLen(ql, address, params):
    string = params["pbstr"]
    if string != 0:
        return len(string)
    return 0


# INT SysReAllocStringLen(
#   BSTR          *pbstr,
#   const OLECHAR *psz,
#   unsigned int  len
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_SysReAllocStringLen(ql, address, params):
    content = params["psz"]
    size = params["len"]
    addr = ql.os.heap.alloc(size + 1)
    ql.mem.write(addr, content[:size].encode("utf-16le"))
    ql.mem.write(params["pbstr"], addr.to_bytes(ql.pointersize, byteorder="little"))
    return 1


# BSTR SysAllocString(
#   const OLECHAR *psz
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_SysAllocString(ql, address, params):
    string = params["psz"]
    if string == 0:
        return 0
    size = len(string)
    addr = ql.os.heap.alloc(size + 1)
    ql.mem.write(addr, string.encode("utf-16le"))
    return addr



from abc import ABC

from qiling.os.memory import QlMemoryHeap
from qiling.os.uefi.utils import init_struct, str_to_guid
from qiling.os.uefi.UefiSpec import EFI_CONFIGURATION_TABLE, EFI_SYSTEM_TABLE
from qiling.os.uefi.smst import EFI_SMM_SYSTEM_TABLE2

class UefiContext(ABC):
	def __init__(self, ql):
		self.ql = ql
		self.heap = None
		self.protocols = {}

		# These members must be initialized before attempting to install a configuration table.
		self.conf_table_array = []
		self.conf_table_array_ptr = 0
		self.conf_table_data_ptr = 0
		self.conf_table_data_next_ptr = 0

	def init_heap(self, base, size):
		self.heap = QlMemoryHeap(self.ql, base, base + size)

	def init_stack(self, base, size):
		self.ql.mem.map(base, size)

	def install_protocol(self, proto_desc, handle, address=None):
		guid = proto_desc['guid']

		if handle not in self.protocols:
			self.protocols[handle] = {}

		if guid in self.protocols[handle]:
			self.ql.log.warning(f'a protocol with guid {guid} is already installed')

		if address is None:
			struct_class = proto_desc['struct']
			address = self.heap.alloc(struct_class.sizeof())

		instance = init_struct(self.ql, address, proto_desc)
		instance.saveTo(self.ql, address)

		self.protocols[handle][guid] = address

	def install_configuration_table(self, guid, table):
		guid = guid.lower()
		confs = self.conf_table_array

		# find configuration table entry by guid. if found, idx would be set to the entry index
		# in the array. if not, idx would be set to one past end of array
		if guid not in confs:
			confs.append(guid)
			
		idx = confs.index(guid)
		ptr = self.conf_table_array_ptr + (idx * EFI_CONFIGURATION_TABLE.sizeof())

		instance = EFI_CONFIGURATION_TABLE()
		instance.VendorGuid = str_to_guid(guid)
		instance.VendorTable = table
		instance.saveTo(self.ql, ptr)

class DxeContext(UefiContext):
	def install_configuration_table(self, guid, table):
		super().install_configuration_table(guid, table)
		# Update number of configuration table entries in the ST.
		gST = EFI_SYSTEM_TABLE.loadFrom(self.ql, self.ql.loader.gST)
		gST.NumberOfTableEntries = len(self.conf_table_array)
		gST.saveTo(self.ql, self.ql.loader.gST)

class SmmContext(UefiContext):
	def __init__(self, ql):
		super(SmmContext, self).__init__(ql)

		# assume tseg is inaccessible to non-smm
		self.tseg_open = False

		# assume tseg is locked
		self.tseg_locked = True

		self.swsmi_handlers = []

	def install_configuration_table(self, guid, table):
		super().install_configuration_table(guid, table)
		# Update number of configuration table entries in the SMST.
		gSmst = EFI_SMM_SYSTEM_TABLE2.loadFrom(self.ql, self.ql.loader.gSmst)
		gSmst.NumberOfTableEntries = len(self.conf_table_array)
		gSmst.saveTo(self.ql, self.ql.loader.gSmst)



#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

# task class not finished 
# TODO: finished
class MachoTask():
    def __init__(self):
        self.id = 9876                  # random id
        self.min_offset = 0x00000000
        self.max_offset = 0xFFE00000


#!/usr/bin/env python3
#
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

from typing import Sequence
from pefile import PE

from qiling.const import QL_ARCH
from qiling.exception import QlErrorArch, QlMemoryMappedError
from qiling.loader.loader import QlLoader
from qiling.os.memory import QlMemoryHeap

from qiling.os.uefi import context, st, smst
from qiling.os.uefi.ProcessorBind import CPU_STACK_ALIGNMENT
from qiling.os.uefi.shutdown import hook_EndOfExecution
from qiling.os.uefi.protocols import EfiLoadedImageProtocol
from qiling.os.uefi.protocols import EfiSmmAccess2Protocol
from qiling.os.uefi.protocols import EfiSmmBase2Protocol
from qiling.os.uefi.protocols import EfiSmmCpuProtocol
from qiling.os.uefi.protocols import EfiSmmSwDispatch2Protocol
from qiling.os.uefi.protocols import PcdProtocol


class QlLoaderPE_UEFI(QlLoader):
    def __init__(self, ql):
        super(QlLoaderPE_UEFI, self).__init__(ql)
        self.ql = ql
        self.modules = []
        self.events = {}
        self.notify_list = []
        self.next_image_base = 0

    # list of members names to save and restore
    __save_members = (
        'modules',
        'events',
        'notify_list',
        'next_image_base',
        'loaded_image_protocol_modules',
        'tpl',
        'efi_conf_table_array',
        'efi_conf_table_array_ptr',
        'efi_conf_table_data_ptr',
        'efi_conf_table_data_next_ptr'
    )

    def save(self) -> dict:
        saved_state = super(QlLoaderPE_UEFI, self).save()

        for member in QlLoaderPE_UEFI.__save_members:
            saved_state[member] = getattr(self, member)

        # since this class initialize the heap (that is hosted by the OS object), we will store it here.
        saved_state['heap'] = self.ql.os.heap.save()

        return saved_state

    def restore(self, saved_state: dict):
        super(QlLoaderPE_UEFI, self).restore(saved_state)

        for member in QlLoaderPE_UEFI.__save_members:
            setattr(self, member, saved_state[member])

        self.ql.os.heap.restore(saved_state['heap'])

    def install_loaded_image_protocol(self, image_base, image_size):
        fields = {
            'gST'        : self.gST,
            'image_base' : image_base,
            'image_size' : image_size
        }

        descriptor = EfiLoadedImageProtocol.make_descriptor(fields)
        self.dxe_context.install_protocol(descriptor, image_base)

        self.loaded_image_protocol_modules.append(image_base)

    def map_and_load(self, path: str, exec_now: bool=False):
        """Map and load a module into memory.

        The specified module would be mapped and loaded into the address set
        in the `next_image_base` member. It is the caller's responsibility to
        make sure that the memory is available.

        On success, `next_image_base` will be updated accordingly.

        Args:
            path     : path of the module binary to load
            exec_now : execute module right away; will be enququed if not

        Raises:
            QlMemoryMappedError : when `next_image_base` is not available
        """

        ql = self.ql
        pe = PE(path, fast_load=True)

        # use image base only if it does not point to NULL
        image_base = pe.OPTIONAL_HEADER.ImageBase or self.next_image_base
        image_size = ql.mem.align(pe.OPTIONAL_HEADER.SizeOfImage, 0x1000)

        assert (image_base % 0x1000) == 0, 'image base is expected to be page-aligned'

        if image_base != pe.OPTIONAL_HEADER.ImageBase:
            pe.relocate_image(image_base)

        pe.parse_data_directories()
        data = bytes(pe.get_memory_mapped_image())

        ql.mem.map(image_base, image_size, info="[module]")
        ql.mem.write(image_base, data)
        ql.log.info(f'Module {path} loaded to {image_base:#x}')

        entry_point = image_base + pe.OPTIONAL_HEADER.AddressOfEntryPoint
        ql.log.info(f'Module entry point at {entry_point:#x}')

        # the 'entry_point' member is used by the debugger. if not set, set it
        # to the first loaded module entry point so the debugger can break
        if self.entry_point == 0:
            self.entry_point = entry_point

        self.install_loaded_image_protocol(image_base, image_size)

        # this would be used later be os.find_containing_image
        self.images.append(self.coverage_image(image_base, image_base + image_size, path))

        # update next memory slot to allow sequencial loading. its availability
        # is unknown though
        self.next_image_base = image_base + image_size

        module_info = (path, image_base, entry_point)

        # execute the module right away or enqueue it
        if exec_now:
            # call entry point while retaining the current return address
            self.execute_module(*module_info, eoe_trap=None)
        else:
            self.modules.append(module_info)

    def call_function(self, addr: int, args: Sequence[int], ret: int):
        """Call a function after properly setting up its arguments and return address.

        Args:
            addr : function address
            args : a sequence of arguments to pass to the function; may be empty
            ret  : return address; may be None
        """

        # arguments gpr (ms x64 cc)
        regs = ('rcx', 'rdx', 'r8', 'r9')
        assert len(args) <= len(regs), f'currently supporting up to {len(regs)} arguments'

        # set up the arguments
        for reg, arg in zip(regs, args):
            self.ql.reg.write(reg, arg)

        # if provided, set return address
        if ret is not None:
            self.ql.stack_push(ret)

        self.ql.reg.rip = addr

    def unload_modules(self):
        for handle in self.loaded_image_protocol_modules:
            struct_addr = self.dxe_context.protocols[handle][self.loaded_image_protocol_guid]
            loaded_image_protocol = EfiLoadedImageProtocol.EFI_LOADED_IMAGE_PROTOCOL.loadFrom(self.ql, struct_addr)

            unload_ptr = self.ql.unpack64(loaded_image_protocol.Unload)

            if unload_ptr != 0:
                self.ql.log.info(f'Unloading module {handle:#x}, calling {unload_ptr:#x}')

                self.call_function(unload_ptr, [handle], self.end_of_execution_ptr)
                self.loaded_image_protocol_modules.remove(handle)

                return True

        return False

    def execute_module(self, path: str, image_base: int, entry_point: int, eoe_trap: int):
        """Start the execution of a UEFI module.

        Args:
            image_base  : module base address
            entry_point : module entry point address
            eoe_trap    : end-of-execution trap address; may be None
        """

        # use familiar UEFI names
        ImageHandle = image_base
        SystemTable = self.gST

        self.call_function(entry_point, [ImageHandle, SystemTable], eoe_trap)
        self.ql.os.entry_point = entry_point

        self.ql.log.info(f'Running from {entry_point:#010x} of {path}')

    def execute_next_module(self):
        if not self.modules or self.ql.os.notify_before_module_execution(self.ql, self.modules[0][0]):
            return

        path, image_base, entry_point = self.modules.pop(0)
        self.execute_module(path, image_base, entry_point, self.end_of_execution_ptr)

    def run(self):
        # intel architecture uefi implementation only
        if self.ql.archtype not in (QL_ARCH.X86, QL_ARCH.X8664):
            raise QlErrorArch("Unsupported architecture")

        # x86-64 arch only
        if self.ql.archtype != QL_ARCH.X8664:
            raise QlErrorArch("Only 64 bit arch is supported at the moment")

        self.loaded_image_protocol_guid = self.ql.os.profile["LOADED_IMAGE_PROTOCOL"]["Guid"]
        self.loaded_image_protocol_modules = []
        self.tpl = 4 # TPL_APPLICATION

        self.user_defined_api = self.ql.os.user_defined_api
        self.user_defined_api_onenter = self.ql.os.user_defined_api_onenter
        self.user_defined_api_onexit = self.ql.os.user_defined_api_onexit

        arch_key = {
            QL_ARCH.X86   : "OS32",
            QL_ARCH.X8664 : "OS64"
        }[self.ql.archtype]

        # -------- init BS / RT / DXE data structures and protocols --------

        os_profile = self.ql.os.profile[arch_key]
        self.dxe_context = context.DxeContext(self.ql)

        # initialize and locate heap
        heap_base = int(os_profile["heap_address"], 0)
        heap_size = int(os_profile["heap_size"], 0)
        self.dxe_context.init_heap(heap_base, heap_size)
        self.heap_base_address = heap_base
        self.ql.log.info(f"Located heap at {heap_base:#010x}")

        # initialize and locate stack
        stack_base = int(os_profile["stack_address"], 0)
        stack_size = int(os_profile["stack_size"], 0)
        self.dxe_context.init_stack(stack_base, stack_size)
        sp = stack_base + stack_size - CPU_STACK_ALIGNMENT
        self.ql.log.info(f"Located stack at {sp:#010x}")

        # TODO: statically allocating 256 KiB for ST, RT, BS, DS and Configuration Tables.
        # however, this amount of memory is rather arbitrary
        gST = self.dxe_context.heap.alloc(256 * 1024)
        st.initialize(self.ql, gST)

        protocols = (
            EfiSmmAccess2Protocol,
            EfiSmmBase2Protocol,
        )

        for proto in protocols:
            self.dxe_context.install_protocol(proto.descriptor, 1)

        # workaround
        self.ql.os.heap = self.dxe_context.heap

        # -------- init SMM data structures and protocols --------

        smm_profile = self.ql.os.profile['SMRAM']
        self.smm_context = context.SmmContext(self.ql)

        # initialize and locate SMM heap
        heap_base = int(smm_profile["heap_address"], 0)
        heap_size = int(smm_profile["heap_size"], 0)
        self.smm_context.init_heap(heap_base, heap_size)
        self.ql.log.info(f"Located SMM heap at {heap_base:#010x}")

        # TODO: statically allocating 256 KiB for SMM ST.
        # however, this amount of memory is rather arbitrary
        gSmst = self.smm_context.heap.alloc(256 * 1024)
        smst.initialize(self.ql, gSmst)

        self.in_smm = False

        protocols = (
            EfiSmmCpuProtocol,
            EfiSmmSwDispatch2Protocol
        )

        for proto in protocols:
            self.smm_context.install_protocol(proto.descriptor, 1)

        # map mmio ranges
        # TODO: move to somehwere more appropriate (+ hook accesses?)
        mmio_map = self.ql.os.profile["MMIO"]
        self.ql.mem.map(
            int(mmio_map['sbreg_base'], 0),
            int(mmio_map['sbreg_size'], 0)
        )

        # set stack and frame pointers
        self.ql.reg.rsp = sp
        self.ql.reg.rbp = sp

        self.entry_point = 0
        self.load_address = 0
        self.next_image_base = int(os_profile["image_address"], 0)

        try:
            for dependency in self.ql.argv:
                self.map_and_load(dependency)
        except QlMemoryMappedError:
            self.ql.log.critical("Couldn't map dependency")

        self.ql.log.info(f"Done with loading {self.ql.path}")

        # set up an end-of-execution hook to regain control when module is done
        # executing (i.e. when the entry point function returns). that should be
        # set on a non-executable address, so SystemTable's address was picked
        self.end_of_execution_ptr = gST
        self.ql.hook_address(hook_EndOfExecution, self.end_of_execution_ptr)

        self.execute_next_module()

    def restore_runtime_services(self):
        pass # not sure why do we need to restore RT



#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

from qiling.const import *
from .fncc import *

DWORD = 1
LOCALE_T = 1
UINT = 1
ULONG = 1
INT = 1
BOOL = 1
SIZE_T = 1
BYTE = 1
CCHAR = 1
CHAR = 1
UCHAR = 1
USHORT = 1
SHORT = 1
ULONGLONG = 2
HANDLE = 3
POINTER = 3
STRING = 4
PCANSI_STRING = 4
PANSI_STRING = 4
PCSZ = 4
WSTRING = 5
PCWSTR = 5
PUNICODE_STRING = 6
PCUNICODE_STRING = 6
WCHAR = 7
BOOLEAN = 8
BOOL = 8
STRING_ADDR = 9
WSTRING_ADDR = 10
GUID = 11



#!/usr/bin/env python3
#
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

import types

from qiling.os.macos.kernel_api import *
from qiling.exception import *


# hook MacOS kernel API
def hook_kernel_api(ql, address, size):
    # call kernel api
    if address in ql.loader.import_symbols:
        api_name = ql.loader.import_symbols[address].decode()
        # print("OK, found hook for %s at 0x%x" % (api_name, address))
        api_func = None

        if api_name in ql.os.user_defined_api:
            if isinstance(ql.os.user_defined_api[api_name], types.FunctionType):
                api_func = ql.os.user_defined_api[api_name]
        else:
            try:
                api_func = globals()['hook_' + api_name]
            except KeyError:
                api_func = None

        if api_func:
            try:
                api_func(ql, address, {})
            except UcError:
                raise
            except Exception:
                ql.log.exception("")
                ql.log.debug("%s Exception Found" % api_name)
                raise QlErrorSyscallError("MacOS kernel API Implementation Error")
        else:
            ql.log.info("%s is not implemented\n" % api_name)
            if ql.debug_stop:
                raise QlErrorSyscallNotFound("MacOS kernel API Implementation Not Found")





#!/usr/bin/env python3
#
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#
import struct
import base64

from qiling.os.windows.fncc import *
from qiling.os.const import *
from qiling.os.windows.utils import *
from qiling.os.windows.handle import *
from qiling.os.windows.const import *

dllname = 'crypt32_dll'

# BOOL CryptStringToBinaryA(
#   LPCSTR pszString,
#   DWORD  cchString,
#   DWORD  dwFlags,
#   BYTE   *pbBinary,
#   DWORD  *pcbBinary,
#   DWORD  *pdwSkip,
#   DWORD  *pdwFlags
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params_type={'BYTE': 'POINTER'},
           replace_params={"pcbBinary": POINTER, "pdwSkip": POINTER, "pdwFlags": POINTER})
def hook_CryptStringToBinaryA(ql, address, params):
    flag_src = params["dwFlags"]
    string_src = params["pszString"]
    size_src = params["cchString"]
    size_dst_pointer = params["pcbBinary"]
    string_dst = params["pbBinary"]
    flag_dst = params["pdwFlags"]

    size_dst = int.from_bytes(ql.mem.read(size_dst_pointer, 4), byteorder="little")
    if size_dst != 0 and size_dst < size_src:
        raise QlErrorNotImplemented("API not implemented")
    if flag_src == CRYPT_STRING_BASE64:
        # Had a padding error, hope this always works
        add_pad = 4 - (len(string_src) % 4)
        if add_pad != 4:
            string_src += "=" * add_pad
        output = base64.b64decode(string_src).decode("utf-16le") + "\x00"
    else:
        ql.log.debug("Flag")
        ql.log.debug(flag_src)
        raise QlErrorNotImplemented("API not implemented")

    if string_dst == 0:
        # Only wants the length
        return len(output)
    else:
        if flag_dst != 0:
            # Is optional
            ql.mem.write(flag_dst, flag_src.to_bytes(length=4, byteorder='little'))
        # Write size
        ql.mem.write(size_dst_pointer, len(output).to_bytes(length=4, byteorder='little'))
        # Write result
        ql.mem.write(string_dst, bytes(output, encoding="utf-16le"))
    return 1


@winsdkapi(cc=STDCALL, dllname=dllname, replace_params_type={'BYTE': 'POINTER'},
           replace_params={"pcbBinary": POINTER, "pdwSkip": POINTER, "pdwFlags": POINTER})
def hook_CryptStringToBinaryW(ql, address, params):
    return hook_CryptStringToBinaryA.__wrapped__(ql, address, params)


#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

import binascii

from uuid import UUID
from typing import Optional

from qiling.os.uefi.const import EFI_SUCCESS, EFI_INVALID_PARAMETER
from qiling.os.uefi.UefiSpec import EFI_CONFIGURATION_TABLE, EFI_SYSTEM_TABLE
from qiling.os.uefi.UefiBaseType import EFI_GUID

def signal_event(ql, event_id: int) -> None:
	event = ql.loader.events[event_id]

	if not event["Set"]:
		event["Set"] = True
		notify_func = event["NotifyFunction"]
		notify_context = event["NotifyContext"]
		ql.loader.notify_list.append((event_id, notify_func, notify_context))

def check_and_notify_protocols(ql) -> bool:
	if ql.loader.notify_list:
		event_id, notify_func, notify_context = ql.loader.notify_list.pop(0)
		ql.log.info(f'Notify event:{event_id} calling:{notify_func:x} context:{notify_context:x}')

		ql.loader.call_function(notify_func, [notify_context], ql.loader.end_of_execution_ptr)

		return True

	return False

def ptr_read8(ql, addr: int) -> int:
	"""Read BYTE data from a pointer
	"""

	return ql.unpack8(ql.mem.read(addr, 1))

def ptr_write8(ql, addr: int, val: int) -> None:
	"""Write BYTE data to a pointer
	"""

	ql.mem.write(addr, ql.pack8(val))

def ptr_read32(ql, addr: int) -> int:
	"""Read DWORD data from a pointer
	"""

	return ql.unpack32(ql.mem.read(addr, 4))

def ptr_write32(ql, addr: int, val: int) -> None:
	"""Write DWORD data to a pointer
	"""

	ql.mem.write(addr, ql.pack32(val))

def ptr_read64(ql, addr: int) -> int:
	"""Read QWORD data from a pointer
	"""

	return ql.unpack64(ql.mem.read(addr, 8))

def ptr_write64(ql, addr: int, val: int) -> None:
	"""Write QWORD data to a pointer
	"""

	ql.mem.write(addr, ql.pack64(val))

# backward comptability
read_int8   = ptr_read8
write_int8  = ptr_write8
read_int32  = ptr_read32
write_int32 = ptr_write32
read_int64  = ptr_read64
write_int64 = ptr_write64

def init_struct(ql, base: int, descriptor: dict):
	struct_class = descriptor['struct']
	struct_fields = descriptor.get('fields', [])

	isntance = struct_class()
	ql.log.info(f'Initializing {struct_class.__name__}')

	for name, value in struct_fields:
		if value is not None:
			# a method: hook this field
			if callable(value):
				p = base + struct_class.offsetof(name)

				setattr(isntance, name, p)
				ql.hook_address(value, p)

				ql.log.info(f' | {name:36s} {p:#010x}')

			# a value: set it
			else:
				setattr(isntance, name, value)

	ql.log.info(f'')

	return isntance

def str_to_guid(guid: str) -> EFI_GUID:
	"""Construct an EFI_GUID structure out of a plain GUID string.
	"""

	buff = UUID(hex=guid).bytes_le

	return EFI_GUID.from_buffer_copy(buff)

def install_configuration_table(context, key: str, table: int):
	"""Create a new Configuration Table entry and add it to the list.

	Args:
		ql    : Qiling instance
		key   : profile section name that holds the entry data
		table : address of configuration table data; if None, data will be read
		        from profile section into memory
	"""

	cfgtable = context.ql.os.profile[key]
	guid = cfgtable['Guid']

	# if pointer to table data was not specified, load table data
	# from profile and have table pointing to it
	if table is None:
		data = binascii.unhexlify(cfgtable['TableData'])
		table = context.conf_table_data_next_ptr

		context.ql.mem.write(table, data)
		context.conf_table_data_next_ptr += len(data)

	context.install_configuration_table(guid, table)

def GetEfiConfigurationTable(context, guid: str) -> Optional[int]:
	"""Find a configuration table by its GUID.
	"""

	guid = guid.lower()
	confs = context.conf_table_array

	if guid in confs:
		idx = confs.index(guid)
		ptr = context.conf_table_array_ptr + (idx * EFI_CONFIGURATION_TABLE.sizeof())

		return ptr

	# not found
	return None



#!/usr/bin/env python3
#
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

import types
from multiprocessing import Process


from qiling.const import *
from qiling.os.linux.thread import *
from qiling.const import *
from qiling.os.posix.filestruct import *
from qiling.os.filestruct import *
from qiling.os.posix.const_mapping import *
from qiling.exception import *

def ql_syscall_clone(ql, clone_flags, clone_child_stack, clone_parent_tidptr, clone_newtls, clone_child_tidptr, *args, **kw):
   
    CSIGNAL = 0x000000ff	
    CLONE_VM = 0x00000100	
    CLONE_FS = 0x00000200	
    CLONE_FILES = 0x00000400	
    CLONE_SIGHAND = 0x00000800	
    CLONE_PIDFD = 0x00001000	
    CLONE_PTRACE = 0x00002000	
    CLONE_VFORK = 0x00004000	
    CLONE_PARENT = 0x00008000	
    CLONE_THREAD = 0x00010000	
    CLONE_NEWNS = 0x00020000	
    CLONE_SYSVSEM = 0x00040000	
    CLONE_SETTLS = 0x00080000	
    CLONE_PARENT_SETTID = 0x00100000	
    CLONE_CHILD_CLEARTID = 0x00200000	
    CLONE_DETACHED = 0x00400000	
    CLONE_UNTRACED = 0x00800000	
    CLONE_CHILD_SETTID = 0x01000000	
    CLONE_NEWCGROUP = 0x02000000	
    CLONE_NEWUTS = 0x04000000	
    CLONE_NEWIPC = 0x08000000	
    CLONE_NEWUSER = 0x10000000	
    CLONE_NEWPID = 0x20000000	
    CLONE_NEWNET = 0x40000000	
    CLONE_IO = 0x80000000

    # X8664 clone_flags, clone_child_stack, clone_parent_tidptr, clone_child_tidptr, clone_newtls
    if ql.archtype== QL_ARCH.X8664:
        ori_clone_newtls = clone_child_tidptr
        clone_child_tidptr = clone_newtls
        clone_newtls = ori_clone_newtls

    if ql.archtype== QL_ARCH.MIPS:
        clone_child_tidptr = ql.unpack32(ql.mem.read(clone_child_tidptr, 4))

    f_th = ql.os.thread_management.cur_thread	
    newtls = None
    set_child_tid_addr = None

    # Shared virtual memory
    if clone_flags & CLONE_VM != CLONE_VM:
        # FIXME: need a proper os.fork() for Windows
        if ql.platform == QL_OS.WINDOWS:
            try:
                pid = Process()
                pid = 0 
            except:
                pid = -1  
        else:
            pid = os.fork()

        if pid == 0:
            ql.os.child_processes = True

            f_th.update_global_thread_id()
            f_th.new_thread_id()

            if clone_flags & CLONE_SETTLS == CLONE_SETTLS:
                f_th.set_thread_tls(clone_newtls)

            if clone_flags & CLONE_CHILD_CLEARTID == CLONE_CHILD_CLEARTID:
                f_th.set_clear_child_tid_addr(clone_child_tidptr)

            if clone_child_stack != 0:
                ql.arch.set_sp(clone_child_stack)
            regreturn = 0
        else:
            regreturn = pid
        
        ql.log.debug("clone(new_stack = %x, flags = %x, tls = %x, ptidptr = %x, ctidptr = %x) = %d" % (clone_child_stack, clone_flags, clone_newtls, clone_parent_tidptr, clone_child_tidptr, regreturn))

        ql.emu_stop()
        return regreturn

    if clone_flags & CLONE_CHILD_SETTID == CLONE_CHILD_SETTID:
        set_child_tid_addr = clone_child_tidptr

    th = ql.os.thread_class.spawn(ql, ql.reg.arch_pc + 2, ql.os.exit_point, set_child_tid_addr = set_child_tid_addr)
    th.current_path = f_th.current_path
    ql.log.debug(f"{str(th)} created.")

    if clone_flags & CLONE_PARENT_SETTID == CLONE_PARENT_SETTID:
        ql.mem.write(clone_parent_tidptr, ql.pack32(th.id))

    ctx = ql.save(reg=True, mem=False)
    # Whether to set a new tls
    if clone_flags & CLONE_SETTLS == CLONE_SETTLS:
        ql.log.debug(f"new_tls={hex(clone_newtls)}")
        th.set_thread_tls(clone_newtls)

    if clone_flags & CLONE_CHILD_CLEARTID == CLONE_CHILD_CLEARTID:
        th.set_clear_child_tid_addr(clone_child_tidptr)

    # Set the stack and return value of the new thread
    # (the return value of the child thread is 0, and the return value of the parent thread is the tid of the child thread)
    # and save the current context.
    regreturn = 0
    ql.reg.arch_sp = clone_child_stack

    # We have to find next pc manually for some archs since the pc is current instruction (like `syscall`).
    if ql.archtype in (QL_ARCH.X8664, ):
        ql.reg.arch_pc += list(ql.disassembler.disasm_lite(bytes(ql.mem.read(ql.reg.arch_pc, 4)), ql.reg.arch_pc))[0][1]
        ql.log.debug(f"Fix pc for child thread to {hex(ql.reg.arch_pc)}")
    ql.os.set_syscall_return(0)
    th.save()
    if th is None or f_th is None:
        raise Exception()
    ql.log.debug("Currently running pid is: %d; tid is: %d " % (
    os.getpid(), ql.os.thread_management.cur_thread.id))
    ql.log.debug("clone(new_stack = %x, flags = %x, tls = %x, ptidptr = %x, ctidptr = %x) = %d" % (
    clone_child_stack, clone_flags, clone_newtls, clone_parent_tidptr, clone_child_tidptr, regreturn))

    # Restore the stack and return value of the parent process
    ql.restore(ctx)
    regreturn = th.id

    # Break the parent process and enter the add new thread event
    ql.emu_stop()
    f_th.stop_event = THREAD_EVENT_CREATE_THREAD
    f_th.stop_return_val = th

    ql.log.debug("Currently running pid is: %d; tid is: %d " % (
    os.getpid(), ql.os.thread_management.cur_thread.id))
    ql.log.debug("clone(new_stack = %x, flags = %x, tls = %x, ptidptr = %x, ctidptr = %x) = %d" % (
    clone_child_stack, clone_flags, clone_newtls, clone_parent_tidptr, clone_child_tidptr, regreturn))

    return regreturn



#!/usr/bin/env python3
#
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

import struct
import time
from qiling.os.windows.const import *
from qiling.os.const import *
from qiling.os.windows.fncc import *
from qiling.os.windows.utils import *
from qiling.os.windows.thread import *
from qiling.os.windows.handle import *
from qiling.exception import *


dllname = 'kernel32_dll'

# BOOL DuplicateHandle(
#   HANDLE   hSourceProcessHandle,
#   HANDLE   hSourceHandle,
#   HANDLE   hTargetProcessHandle,
#   LPHANDLE lpTargetHandle,
#   DWORD    dwDesiredAccess,
#   BOOL     bInheritHandle,
#   DWORD    dwOptions
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params_type={'HANDLE': 'POINTER'})
def hook_DuplicateHandle(ql, address, params):
    # TODO for how we manage handle, i think this doesn't work
    content = params["hSourceHandle"]
    dst = params["lpTargetHandle"]
    ql.mem.write(dst, content.to_bytes(length=ql.pointersize, byteorder='little'))
    return 1


# BOOL CloseHandle(
#   HANDLE hObject
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_CloseHandle(ql, address, params):
    value = params["hObject"]
    handle = ql.os.handle_manager.get(value)
    if handle is None:
        ql.os.last_error = ERROR_INVALID_HANDLE
        return 0
    else:
        if handle.permissions is not None and handle.permissions & HANDLE_FLAG_PROTECT_FROM_CLOSE >= 1:
            # FIXME: add error
            return 0
        else:
            ql.os.handle_manager.delete(value)

    return 1


# BOOL SetHandleInformation(
#   HANDLE hObject,
#   DWORD  dwMask,
#   DWORD  dwFlags
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params_type={'HANDLE': 'POINTER'})
def hook_SetHandleInformation(ql, address, params):
    val = params["hObject"]
    handle = ql.os.handle_manager.get(val)
    handle.permissions = params["dwFlags"]
    return 1



#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

from qiling.os.const import *
from ..const import *
from ..fncc import *
from ..ProcessorBind import *
from ..UefiBaseType import *

# @file: MdePkg\Include\Protocol\SmmSwDispatch2.h
class EFI_SMM_SW_REGISTER_CONTEXT(STRUCT):
	_fields_ = [
		('SwSmiInputValue', UINTN)
	]

# @ file: MdePkg\Include\Pi\PiMmCis.h
EFI_SMM_HANDLER_ENTRY_POINT2 = FUNCPTR(EFI_STATUS, EFI_HANDLE, PTR(VOID), PTR(VOID), PTR(UINTN))

class EFI_SMM_SW_DISPATCH2_PROTOCOL(STRUCT):
	EFI_SMM_SW_DISPATCH2_PROTOCOL = STRUCT

	_fields_ = [
		('Register',		FUNCPTR(EFI_STATUS, PTR(EFI_SMM_SW_DISPATCH2_PROTOCOL), EFI_SMM_HANDLER_ENTRY_POINT2, PTR(EFI_SMM_SW_REGISTER_CONTEXT), PTR(EFI_HANDLE))),
		('UnRegister',		FUNCPTR(EFI_STATUS, PTR(EFI_SMM_SW_DISPATCH2_PROTOCOL), EFI_HANDLE)),
		('MaximumSwiValue',	UINTN)
	]

@dxeapi(params = {
	"This"				: POINTER,
	"DispatchFunction"	: POINTER,
	"RegisterContext"	: POINTER,
	"DispatchHandle"	: POINTER
})
def hook_Register(ql, address, params):
	# Let's save the dispatch params, so they can be triggered if needed. 
	ql.loader.smm_context.swsmi_handlers.append(params)
	return EFI_SUCCESS

@dxeapi(params = {
	"This"				: POINTER,
	"DispatchHandle"	: POINTER
})
def hook_UnRegister(ql, address, params):
	return EFI_SUCCESS

descriptor = {
	"guid" : "18a3c6dc-5eea-48c8-a1c1-b53389f98999",
	"struct" : EFI_SMM_SW_DISPATCH2_PROTOCOL,
	"fields" : (
		("Register",	hook_Register),
		("UnRegister",	hook_UnRegister)
	)
}
